//	Agent.h - Base class for all the agents in the system.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef AGENT_H_
#define AGENT_H_

#include "TwoFloats.h"
#include "c0xHeaders.h"
#include "ThreeFloats.h"

#include <vector>
#include <string>
#include <stdint.h>

class Agent;
class Drawer;
class GamePlay;

typedef StdWeakPtr<Agent>::type AgentWeak;
typedef StdSharedPtr<Agent>::type AgentPtr;

///	Base class for all the agents in the system.
class Agent
{
  public:
	///	The different possible movement states.
	enum MovementState
	{
		Jitter,		//Slight, random-ish movement around a static point.
		Wander,		//Aimless movement.
		MoveTowards,//Move towards a point.
		MoveAway,	//Move away from a point.
		Orbit,		//Orbit a point.
		Follow		//Follow another agent.
	};

	///	Constructor.
	Agent(MovementState initialState);
	///	Destructor.
	virtual ~Agent();

	///	Draws the agent.
	virtual void draw(Drawer *d, uint32_t font, float alpha) = 0;
	///	Called to move the agent.
	void move();

	///	Called when the player tries to move the agent.
	virtual void playerMove(const TwoFloats& pos) = 0;
	///	Called when the player finishes trying to move the agent.
	virtual void endPlayerMove() = 0;

	///	Called when the player adds a letter to the agent.
	virtual void playerAddChar(char character) = 0;

	///	Called when the agent is near another agent.
	virtual void nearby(AgentWeak& other, GamePlay *gamePlay) = 0;

	///	Returns this agent's name/type.
	virtual const std::string getName() const = 0;
	///	Returns this agent's current movement state.
	MovementState getMovementState() const {return movementState;};
	///	Returns this agent's current position.
	virtual const TwoFloats& getPosition() const = 0;
	///	Returns a vector of this agent's recent path through the void.
	virtual const std::vector<TwoFloats>& getPath() const = 0;
	///	Returns this agent's colour.
	virtual const ThreeFloats getColour() const = 0;
	///	Returns this agent's letter.
	virtual char getLetter() const = 0;
	///	Returns this agent's word.
	virtual std::string getWord() const = 0;
	///	Returns true if this agent is dead.
	virtual bool getDead() const = 0;
  protected:
	///	Sets this agent's current movement state.
	void setMovementState(MovementState newState);

	///	Moves the agent slightly around a static point.
	virtual void moveJitter() = 0;
	///	Moves the agent aimlessly.
	virtual void moveWander() = 0;
	///	Moves the agent towards a point.
	virtual void moveTowards() = 0;
	///	Moves the agent away from a point.
	virtual void moveAway() = 0;
	///	Moves the agent in an orbit around a point.
	virtual void moveOrbit() = 0;
	///	Moves the agent towards another.
	virtual void moveFollow() = 0;
  private:
	///	This agent's current movement state.
	MovementState movementState;
};

#endif
