//	GamePlay.h - Main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAY_H_
#define GAMEPLAY_H_

#include "c0xHeaders.h"
#include "Background.h"
#include "GameState.h"
#include "Agent.h"

#include <stdint.h>
#include <sstream>
#include <vector>

class Trail;

///	Main gameplay state.
class GamePlay : public GameState
{
  public:
	///	Constructor.
	GamePlay(Application *app);
	///	Destructor.
	~GamePlay();

	///	Draws the state.
	void draw(Drawer *d);

	///	Called when a key down event is received.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);
	///	Called when a mouse move event is received.
	void mouseMove(int posX, int posY);

	///	Returns the name of this state.
	const std::string getName() const {return "GamePlay";};

	///	Called to let agents 'give birth' to Letters.
	void addLetter(const TwoFloats& pos, char letter, const ThreeFloats& col);
	///	Called to let agents 'give birth' to Words.
	void addWord(const TwoFloats& pos,
				 const std::string& word,
				 const ThreeFloats& col);

	///	Called when a Voiceover reaches a marker.
	void markerCallback(const std::string& marker);
	///	So we know when the cacophony has finished.
	void setEnd();
  private:
	///	Helper method to calculate which Agents are 'nearby' each other.
	void calcNearby();

	///	Used to draw stars at the end.
	struct Star
	{
		///	The star's position.
		TwoFloats position;
		///	The star's size.
		float size;
		///	The star's colour.
		ThreeFloats colour;
	};
	///	Used to draw stars at the end.
	struct StarField
	{
		///	The index of the image to use.
		int image;
		///	The field's position on the screen.
		TwoFloats position;
	};

	///	The tree image.
	static uint32_t treeImage;
	///	Used to draw stars at the end.
	static uint32_t starImage;
	///	Used to draw stars at the end.
	static uint32_t starBlurImage;
	///	Used to give space a texture.
	static uint32_t noiseImage;
	///	Nasa image of Earth.
	static uint32_t earthImage;
	///	Used to draw stars at the end.
	static uint32_t starImages[6];
	///	The font we use for the title.
	static uint32_t bigFont;
	///	The font we use for letters/words/numbers/symbols.
	static uint32_t weeFont;

	///	The background watercolours.
	Background background;

	///	Timer used to ensure nothing happens until after the introduction.
	//int introTimer;
	///	Used to fade in the tree.
	float treeFade;
	///	Used to fade out the title.
	float titleFade;
	///	True if we're currently in the intro.
	bool intro;

	///	All the Agents currently in play.
	std::vector<AgentPtr> agents;

	///	The Trail we're currently adding points to.
	int currentTrail;
	///	Used to add multiple trails.
	int trailCounter;

	///	The Word we're currently adding letters to.
	int currentWord;
	///	Used to determine when to add Letters, and when to add Words.
	int wordCounter;
	///	Used to keep track of the letters we're adding to a Word.
	std::stringstream tempWord;

	///	Used to trigger voiceovers in the correct order.
	bool letterBorn;
	///	Used to trigger voiceovers in the correct order.
	bool wordBorn;
	///	Used to trigger voiceovers in the correct order.
	bool trailBorn;
	///	Used to trigger voiceovers in the correct order.
	std::stringstream agentVoiceover;
	///	How many agent types have been created so far.
	int numAgentTypes;

	///	Used to determine when to play the waiting voiceover hint.
	int waitingCount;
	///	Used to determine when to end the story.
	int endCount;
	///	Used to fade to black.
	float fadeToBlack;

	///	Used at the very end to fade out and quit to desktop.
	float endFade;

	///	The stars at the end.
	std::vector<Star> stars;
	///	The stars at the end.
	StarField starFields[12];
	///	Used to move the earth into view.
	float earthMove;
	///	Used to fade in the ending text.
	float endTextFade;
};

#endif
