//	KeyManager.cpp - Class used to handle configurable keys.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "KeyManager.h"
#include "FilePath.h"
#include "DebugStatement.h"

#include <fstream>

using namespace std;

//------------------------------------------------------------------------------
KeyManager::KeyManager()
{
	
}

//------------------------------------------------------------------------------
KeyManager::~KeyManager()
{
	
}

//------------------------------------------------------------------------------
void KeyManager::addKeypress(const std::string& name, KeyPress defaultKey)
{
	mappings.insert(make_pair(defaultKey, name));
}

//------------------------------------------------------------------------------
void KeyManager::loadConfigFile(const string& appName)
{
	FilePath configFile = FilePath::getDataFilesDir().getChild("keyconfig.txt");

	if(!configFile.exists())
		configFile = FilePath::getAppDataPath().getChild(appName).getChild("keyconfig.txt");

	if(configFile.exists())
	{
		string tempstr;
		string action;
		KeyPress key;
		char buffer[1024];
		ifstream configStream(configFile.getPath().c_str());

		DebugStatement("KeyManager: " << configFile.getPath().c_str() << " loaded.");

		//Get rid of the existing mappings.
		mappings.clear();

		while(!configStream.eof())
		{
			configStream.getline(buffer, 1024);

			tempstr = buffer;
			action = tempstr.substr(0, tempstr.find_first_of(' '));
			key = getKeyPressFromText(tempstr.substr(tempstr.find_first_of(' ')+1));
			mappings.insert(make_pair(key, action));
			DebugStatement("KeyManager: key = " << key << " : action = " << action);
		}
	}
	else
		saveConfigFile(appName);
}

//------------------------------------------------------------------------------
void KeyManager::saveConfigFile(const string& appName)
{
	map<KeyPress, string>::iterator it;
	FilePath configFile = FilePath::getDataFilesDir().getChild("keyconfig.txt");

	if(!configFile.exists())
	{
		FilePath configParent = FilePath::getAppDataPath().getChild(appName);

		if(!configParent.isDirectory())
			configParent.createAsDirectory();

		configFile = FilePath::getAppDataPath().getChild(appName).getChild("keyconfig.txt");
	}

	DebugStatement("KeyManager: Saving keyconfig.txt to: " << configFile.getPath());

	ofstream configStream(configFile.getPath().c_str());

	for(it=mappings.begin();it!=mappings.end();++it)
		configStream << it->second << " " << getKeyPressText(it->first) << endl;
}

//------------------------------------------------------------------------------
const string KeyManager::getAction(KeyPress key)
{
	if(mappings.find(key) == mappings.end())
		return "";
	else
		return mappings[key];
}

//------------------------------------------------------------------------------
const string KeyManager::getAction(const SDL_keysym& keyPressed)
{
	KeyPress key = KeyNull;

	switch(keyPressed.sym)
	{
		case SDLK_a:
			key = KeyA;
			break;
		case SDLK_b:
			key = KeyB;
			break;
		case SDLK_c:
			key = KeyC;
			break;
		case SDLK_d:
			key = KeyD;
			break;
		case SDLK_e:
			key = KeyE;
			break;
		case SDLK_f:
			key = KeyF;
			break;
		case SDLK_g:
			key = KeyG;
			break;
		case SDLK_h:
			key = KeyH;
			break;
		case SDLK_i:
			key = KeyI;
			break;
		case SDLK_j:
			key = KeyJ;
			break;
		case SDLK_k:
			key = KeyK;
			break;
		case SDLK_l:
			key = KeyL;
			break;
		case SDLK_m:
			key = KeyM;
			break;
		case SDLK_n:
			key = KeyN;
			break;
		case SDLK_o:
			key = KeyO;
			break;
		case SDLK_p:
			key = KeyP;
			break;
		case SDLK_q:
			key = KeyQ;
			break;
		case SDLK_r:
			key = KeyR;
			break;
		case SDLK_s:
			key = KeyS;
			break;
		case SDLK_t:
			key = KeyT;
			break;
		case SDLK_u:
			key = KeyU;
			break;
		case SDLK_v:
			key = KeyV;
			break;
		case SDLK_w:
			key = KeyW;
			break;
		case SDLK_x:
			key = KeyX;
			break;
		case SDLK_y:
			key = KeyY;
			break;
		case SDLK_z:
			key = KeyZ;
			break;
		case SDLK_0:
			key = Key0;
			break;
		case SDLK_1:
			key = Key1;
			break;
		case SDLK_2:
			key = Key2;
			break;
		case SDLK_3:
			key = Key3;
			break;
		case SDLK_4:
			key = Key4;
			break;
		case SDLK_5:
			key = Key5;
			break;
		case SDLK_6:
			key = Key6;
			break;
		case SDLK_7:
			key = Key7;
			break;
		case SDLK_8:
			key = Key8;
			break;
		case SDLK_9:
			key = Key9;
			break;
		case SDLK_TAB:
			key = KeyTab;
			break;
		case SDLK_LSHIFT:
		case SDLK_RSHIFT:
			key = KeyShift;
			break;
		case SDLK_LCTRL:
		case SDLK_RCTRL:
			key = KeyCtrl;
			break;
		case SDLK_LALT:
		case SDLK_RALT:
			key = KeyAlt;
			break;
		case SDLK_SPACE:
			key = KeySpace;
			break;
		case SDLK_RETURN:
			key = KeyReturn;
			break;
		case SDLK_UP:
			key = KeyUp;
			break;
		case SDLK_LEFT:
			key = KeyLeft;
			break;
		case SDLK_RIGHT:
			key = KeyRight;
			break;
		case SDLK_DOWN:
			key = KeyDown;
			break;
		case SDLK_HOME:
			key = KeyHome;
			break;
		case SDLK_END:
			key = KeyEnd;
			break;
		case SDLK_PAGEUP:
			key = KeyPgUp;
			break;
		case SDLK_PAGEDOWN:
			key = KeyPgDown;
			break;
		case SDLK_KP0:
			key = KeyNum0;
			break;
		case SDLK_KP1:
			key = KeyNum1;
			break;
		case SDLK_KP2:
			key = KeyNum2;
			break;
		case SDLK_KP3:
			key = KeyNum3;
			break;
		case SDLK_KP4:
			key = KeyNum4;
			break;
		case SDLK_KP5:
			key = KeyNum5;
			break;
		case SDLK_KP6:
			key = KeyNum6;
			break;
		case SDLK_KP7:
			key = KeyNum7;
			break;
		case SDLK_KP8:
			key = KeyNum8;
			break;
		case SDLK_KP9:
			key = KeyNum9;
			break;
		case SDLK_MINUS:
			key = KeyMinus;
			break;
		case SDLK_EQUALS:
			key = KeyEquals;
			break;
		case SDLK_BACKSPACE:
			key = KeyBackspace;
			break;
		case SDLK_DELETE:
			key = KeyDelete;
			break;
		case SDLK_ESCAPE:
			key = KeyEscape;
			break;
		case SDLK_COMMA:
			key = KeyComma;
			break;
		case SDLK_PERIOD:
			key = KeyPeriod;
			break;
	}

	return mappings[key];
}

//------------------------------------------------------------------------------
const string KeyManager::getAction(int joystick, int param)
{
	KeyPress key = Joy0Button1;

	switch(joystick)
	{
		case 0:
			key = (KeyPress)((int)Joy0Up + param);
			break;
		case 1:
			key = (KeyPress)((int)Joy1Up + param);
			break;
		case 2:
			key = (KeyPress)((int)Joy2Up + param);
			break;
		case 3:
			key = (KeyPress)((int)Joy3Up + param);
			break;
	}

	return mappings[key];
}

//------------------------------------------------------------------------------
char KeyManager::keySymToChar(const SDL_keysym& keyPressed)
{
	char retval = ' ';

	if(keyPressed.mod&KMOD_SHIFT)
	{
		switch(keyPressed.sym)
		{
			case SDLK_1:
				retval = '!';
				break;
			case SDLK_2:
				retval = '\"';
				break;
			case SDLK_3:
				retval = '';
				break;
			case SDLK_4:
				retval = '$';
				break;
			case SDLK_5:
				retval = '%';
				break;
			case SDLK_6:
				retval = '^';
				break;
			case SDLK_7:
				retval = '&';
				break;
			case SDLK_8:
				retval = '*';
				break;
			case SDLK_9:
				retval = '(';
				break;
			case SDLK_0:
				retval = ')';
				break;
			case SDLK_a:
				retval = 'A';
				break;
			case SDLK_b:
				retval = 'B';
				break;
			case SDLK_c:
				retval = 'C';
				break;
			case SDLK_d:
				retval = 'D';
				break;
			case SDLK_e:
				retval = 'E';
				break;
			case SDLK_f:
				retval = 'F';
				break;
			case SDLK_g:
				retval = 'G';
				break;
			case SDLK_h:
				retval = 'H';
				break;
			case SDLK_i:
				retval = 'I';
				break;
			case SDLK_j:
				retval = 'J';
				break;
			case SDLK_k:
				retval = 'K';
				break;
			case SDLK_l:
				retval = 'L';
				break;
			case SDLK_m:
				retval = 'M';
				break;
			case SDLK_n:
				retval = 'N';
				break;
			case SDLK_o:
				retval = 'O';
				break;
			case SDLK_p:
				retval = 'P';
				break;
			case SDLK_q:
				retval = 'Q';
				break;
			case SDLK_r:
				retval = 'R';
				break;
			case SDLK_s:
				retval = 'S';
				break;
			case SDLK_t:
				retval = 'T';
				break;
			case SDLK_u:
				retval = 'U';
				break;
			case SDLK_v:
				retval = 'V';
				break;
			case SDLK_w:
				retval = 'W';
				break;
			case SDLK_x:
				retval = 'X';
				break;
			case SDLK_y:
				retval = 'Y';
				break;
			case SDLK_z:
				retval = 'Z';
			case SDLK_COMMA:
				retval = '<';
				break;
			case SDLK_PERIOD:
				retval = '>';
				break;
			case SDLK_MINUS:
				retval = '_';
				break;
			case SDLK_EQUALS:
				retval = '+';
				break;
			case SDLK_LEFTPAREN:
				retval = '(';
				break;
			case SDLK_RIGHTPAREN:
				retval = ')';
				break;
			case SDLK_SPACE:
				retval = ' ';
				break;
		}
	}
	else
	{
		switch(keyPressed.sym)
		{
			case SDLK_0:
				retval = '0';
				break;
			case SDLK_1:
				retval = '1';
				break;
			case SDLK_2:
				retval = '2';
				break;
			case SDLK_3:
				retval = '3';
				break;
			case SDLK_4:
				retval = '4';
				break;
			case SDLK_5:
				retval = '5';
				break;
			case SDLK_6:
				retval = '6';
				break;
			case SDLK_7:
				retval = '7';
				break;
			case SDLK_8:
				retval = '8';
				break;
			case SDLK_9:
				retval = '9';
				break;
			case SDLK_a:
				retval = 'a';
				break;
			case SDLK_b:
				retval = 'b';
				break;
			case SDLK_c:
				retval = 'c';
				break;
			case SDLK_d:
				retval = 'd';
				break;
			case SDLK_e:
				retval = 'e';
				break;
			case SDLK_f:
				retval = 'f';
				break;
			case SDLK_g:
				retval = 'g';
				break;
			case SDLK_h:
				retval = 'h';
				break;
			case SDLK_i:
				retval = 'i';
				break;
			case SDLK_j:
				retval = 'j';
				break;
			case SDLK_k:
				retval = 'k';
				break;
			case SDLK_l:
				retval = 'l';
				break;
			case SDLK_m:
				retval = 'm';
				break;
			case SDLK_n:
				retval = 'n';
				break;
			case SDLK_o:
				retval = 'o';
				break;
			case SDLK_p:
				retval = 'p';
				break;
			case SDLK_q:
				retval = 'q';
				break;
			case SDLK_r:
				retval = 'r';
				break;
			case SDLK_s:
				retval = 's';
				break;
			case SDLK_t:
				retval = 't';
				break;
			case SDLK_u:
				retval = 'u';
				break;
			case SDLK_v:
				retval = 'v';
				break;
			case SDLK_w:
				retval = 'w';
				break;
			case SDLK_x:
				retval = 'x';
				break;
			case SDLK_y:
				retval = 'y';
				break;
			case SDLK_z:
				retval = 'z';
			case SDLK_COMMA:
				retval = ',';
				break;
			case SDLK_PERIOD:
				retval = '.';
				break;
			case SDLK_MINUS:
				retval = '-';
				break;
			case SDLK_EQUALS:
				retval = '=';
				break;
			case SDLK_LEFTPAREN:
				retval = '(';
				break;
			case SDLK_RIGHTPAREN:
				retval = ')';
				break;
			case SDLK_SPACE:
				retval = ' ';
				break;
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
string KeyManager::getKeyPressText(KeyPress key) const
{
	string retval;

	switch(key)
	{
		case KeyA:
			return "a";
		case KeyB:
			return "b";
		case KeyC:
			return "c";
		case KeyD:
			return "d";
		case KeyE:
			return "e";
		case KeyF:
			return "f";
		case KeyG:
			return "g";
		case KeyH:
			return "h";
		case KeyI:
			return "i";
		case KeyJ:
			return "j";
		case KeyK:
			return "k";
		case KeyL:
			return "l";
		case KeyM:
			return "m";
		case KeyN:
			return "n";
		case KeyO:
			return "o";
		case KeyP:
			return "p";
		case KeyQ:
			return "q";
		case KeyR:
			return "r";
		case KeyS:
			return "s";
		case KeyT:
			return "t";
		case KeyU:
			return "u";
		case KeyV:
			return "v";
		case KeyW:
			return "w";
		case KeyX:
			return "x";
		case KeyY:
			return "y";
		case KeyZ:
			return "z";
		case Key0:
			return "0";
		case Key1:
			return "1";
		case Key2:
			return "2";
		case Key3:
			return "3";
		case Key4:
			return "4";
		case Key5:
			return "5";
		case Key6:
			return "6";
		case Key7:
			return "7";
		case Key8:
			return "8";
		case Key9:
			return "9";
		case KeyTab:
			return "tab";
		case KeyShift:
			return "shift";
		case KeyCtrl:
			return "ctrl";
		case KeyAlt:
			return "alt";
		case KeySpace:
			return "space";
		case KeyReturn:
			return "return";
		case KeyUp:
			return "up";
		case KeyLeft:
			return "left";
		case KeyRight:
			return "right";
		case KeyDown:
			return "down";
		case KeyHome:
			return "home";
		case KeyEnd:
			return "end";
		case KeyPgUp:
			return "pgup";
		case KeyPgDown:
			return "pgdown";
		case KeyNum0:
			return "num0";
		case KeyNum1:
			return "num1";
		case KeyNum2:
			return "num2";
		case KeyNum3:
			return "num3";
		case KeyNum4:
			return "num4";
		case KeyNum5:
			return "num5";
		case KeyNum6:
			return "num6";
		case KeyNum7:
			return "num7";
		case KeyNum8:
			return "num8";
		case KeyNum9:
			return "num9";
		case KeyMinus:
			return "minus";
		case KeyEquals:
			return "equals";
		case KeyBackspace:
			return "backspace";
		case KeyDelete:
			return "delete";
		case KeyEscape:
			return "escape";
		case KeyComma:
			return "comma";
		case KeyPeriod:
			return "period";
		case Joy0Up:
			return "joy0Up";
		case Joy0Left:
			return "joy0Left";
		case Joy0Right:
			return "joy0Right";
		case Joy0Down:
			return "joy0Down";
		case Joy0Trigger1:
			return "joy0Trigger1";
		case Joy0Trigger2:
			return "joy0Trigger2";
		case Joy0Up2:
			return "joy0Up2";
		case Joy0Left2:
			return "joy0Left2";
		case Joy0Right2:
			return "joy0Right2";
		case Joy0Down2:
			return "joy0Down2";
		case Joy0Button1:
			return "joy0Button1";
		case Joy0Button2:
			return "joy0Button2";
		case Joy0Button3:
			return "joy0Button3";
		case Joy0Button4:
			return "joy0Button4";
		case Joy0Button5:
			return "joy0Button5";
		case Joy0Button6:
			return "joy0Button6";
		case Joy0Button7:
			return "joy0Button7";
		case Joy0Button8:
			return "joy0Button8";
		case Joy0Button9:
			return "joy0Button9";
		case Joy0Button10:
			return "joy0Button10";
		case Joy1Up:
			return "joy1Up";
		case Joy1Left:
			return "joy1Left";
		case Joy1Right:
			return "joy1Right";
		case Joy1Down:
			return "joy1Down";
		case Joy1Trigger1:
			return "joy1Trigger1";
		case Joy1Trigger2:
			return "joy1Trigger2";
		case Joy1Up2:
			return "joy1Up2";
		case Joy1Left2:
			return "joy1Left2";
		case Joy1Right2:
			return "joy1Right2";
		case Joy1Down2:
			return "joy1Down2";
		case Joy1Button1:
			return "joy1Button1";
		case Joy1Button2:
			return "joy1Button2";
		case Joy1Button3:
			return "joy1Button3";
		case Joy1Button4:
			return "joy1Button4";
		case Joy1Button5:
			return "joy1Button5";
		case Joy1Button6:
			return "joy1Button6";
		case Joy1Button7:
			return "joy1Button7";
		case Joy1Button8:
			return "joy1Button8";
		case Joy1Button9:
			return "joy1Button9";
		case Joy1Button10:
			return "joy1Button10";
		case Joy2Up:
			return "joy2Up";
		case Joy2Left:
			return "joy2Left";
		case Joy2Right:
			return "joy2Right";
		case Joy2Down:
			return "joy2Down";
		case Joy2Trigger1:
			return "joy2Trigger1";
		case Joy2Trigger2:
			return "joy2Trigger2";
		case Joy2Up2:
			return "joy2Up2";
		case Joy2Left2:
			return "joy2Left2";
		case Joy2Right2:
			return "joy2Right2";
		case Joy2Down2:
			return "joy2Down2";
		case Joy2Button1:
			return "joy2Button1";
		case Joy2Button2:
			return "joy2Button2";
		case Joy2Button3:
			return "joy2Button3";
		case Joy2Button4:
			return "joy2Button4";
		case Joy2Button5:
			return "joy2Button5";
		case Joy2Button6:
			return "joy2Button6";
		case Joy2Button7:
			return "joy2Button7";
		case Joy2Button8:
			return "joy2Button8";
		case Joy2Button9:
			return "joy2Button9";
		case Joy2Button10:
			return "joy2Button10";
		case Joy3Up:
			return "joy3Up";
		case Joy3Left:
			return "joy3Left";
		case Joy3Right:
			return "joy3Right";
		case Joy3Down:
			return "joy3Down";
		case Joy3Trigger1:
			return "joy3Trigger1";
		case Joy3Trigger2:
			return "joy3Trigger2";
		case Joy3Up2:
			return "joy3Up2";
		case Joy3Left2:
			return "joy3Left2";
		case Joy3Right2:
			return "joy3Right2";
		case Joy3Down2:
			return "joy3Down2";
		case Joy3Button1:
			return "joy3Button1";
		case Joy3Button2:
			return "joy3Button2";
		case Joy3Button3:
			return "joy3Button3";
		case Joy3Button4:
			return "joy3Button4";
		case Joy3Button5:
			return "joy3Button5";
		case Joy3Button6:
			return "joy3Button6";
		case Joy3Button7:
			return "joy3Button7";
		case Joy3Button8:
			return "joy3Button8";
		case Joy3Button9:
			return "joy3Button9";
		case Joy3Button10:
			return "joy3Button10";
	}

	return retval;
}

//------------------------------------------------------------------------------
KeyManager::KeyPress KeyManager::getKeyPressFromText(const string& text) const
{
	KeyPress retval;

	if(text == "a")
		retval = KeyA;
	if(text == "b")
		retval = KeyB;
	if(text == "c")
		retval = KeyC;
	if(text == "d")
		retval = KeyD;
	if(text == "e")
		retval = KeyE;
	if(text == "f")
		retval = KeyF;
	if(text == "g")
		retval = KeyG;
	if(text == "h")
		retval = KeyH;
	if(text == "i")
		retval = KeyI;
	if(text == "j")
		retval = KeyJ;
	if(text == "k")
		retval = KeyK;
	if(text == "l")
		retval = KeyL;
	if(text == "m")
		retval = KeyM;
	if(text == "n")
		retval = KeyN;
	if(text == "o")
		retval = KeyO;
	if(text == "p")
		retval = KeyP;
	if(text == "q")
		retval = KeyQ;
	if(text == "r")
		retval = KeyR;
	if(text == "s")
		retval = KeyS;
	if(text == "t")
		retval = KeyT;
	if(text == "u")
		retval = KeyU;
	if(text == "v")
		retval = KeyV;
	if(text == "w")
		retval = KeyW;
	if(text == "x")
		retval = KeyX;
	if(text == "y")
		retval = KeyY;
	if(text == "z")
		retval = KeyZ;
	if(text == "0")
		retval = Key0;
	if(text == "1")
		retval = Key1;
	if(text == "2")
		retval = Key2;
	if(text == "3")
		retval = Key3;
	if(text == "4")
		retval = Key4;
	if(text == "5")
		retval = Key5;
	if(text == "6")
		retval = Key6;
	if(text == "7")
		retval = Key7;
	if(text == "8")
		retval = Key8;
	if(text == "9")
		retval = Key9;
	if(text == "tab")
		retval = KeyTab;
	if(text == "shift")
		retval = KeyShift;
	if(text == "ctrl")
		retval = KeyCtrl;
	if(text == "alt")
		retval = KeyAlt;
	if(text == "space")
		retval = KeySpace;
	if(text == "return")
		retval = KeyReturn;
	if(text == "up")
		retval = KeyUp;
	if(text == "left")
		retval = KeyLeft;
	if(text == "right")
		retval = KeyRight;
	if(text == "down")
		retval = KeyDown;
	if(text == "home")
		retval = KeyHome;
	if(text == "end")
		retval = KeyEnd;
	if(text == "pgup")
		retval = KeyPgUp;
	if(text == "pgdown")
		retval = KeyPgDown;
	if(text == "num0")
		retval = KeyNum0;
	if(text == "num1")
		retval = KeyNum1;
	if(text == "num2")
		retval = KeyNum2;
	if(text == "num3")
		retval = KeyNum3;
	if(text == "num4")
		retval = KeyNum4;
	if(text == "num5")
		retval = KeyNum5;
	if(text == "num6")
		retval = KeyNum6;
	if(text == "num7")
		retval = KeyNum7;
	if(text == "num8")
		retval = KeyNum8;
	if(text == "num9")
		retval = KeyNum9;
	if(text == "minus")
		retval = KeyMinus;
	if(text == "equals")
		retval = KeyEquals;
	if(text == "backspace")
		retval = KeyBackspace;
	if(text == "delete")
		retval = KeyDelete;
	if(text == "escape")
		retval = KeyEscape;
	if(text == "comma")
		retval = KeyComma;
	if(text == "period")
		retval = KeyPeriod;
	if(text == "joy0Up")
		retval = Joy0Up;
	if(text == "joy0Left")
		retval = Joy0Left;
	if(text == "joy0Right")
		retval = Joy0Right;
	if(text == "joy0Down")
		retval = Joy0Down;
	if(text == "joy0Trigger1")
		retval = Joy0Trigger1;
	if(text == "joy0Trigger2")
		retval = Joy0Trigger2;
	if(text == "joy0Up2")
		retval = Joy0Up2;
	if(text == "joy0Left2")
		retval = Joy0Left2;
	if(text == "joy0Right2")
		retval = Joy0Right2;
	if(text == "joy0Down2")
		retval = Joy0Down2;
	if(text == "joy0Button1")
		retval = Joy0Button1;
	if(text == "joy0Button2")
		retval = Joy0Button2;
	if(text == "joy0Button3")
		retval = Joy0Button3;
	if(text == "joy0Button4")
		retval = Joy0Button4;
	if(text == "joy0Button5")
		retval = Joy0Button5;
	if(text == "joy0Button6")
		retval = Joy0Button6;
	if(text == "joy0Button7")
		retval = Joy0Button7;
	if(text == "joy0Button8")
		retval = Joy0Button8;
	if(text == "joy0Button9")
		retval = Joy0Button9;
	if(text == "joy0Button10")
		retval = Joy0Button10;
	if(text == "joy1Up")
		retval = Joy1Up;
	if(text == "joy1Left")
		retval = Joy1Left;
	if(text == "joy1Right")
		retval = Joy1Right;
	if(text == "joy1Down")
		retval = Joy1Down;
	if(text == "joy1Trigger1")
		retval = Joy1Trigger1;
	if(text == "joy1Trigger2")
		retval = Joy1Trigger2;
	if(text == "joy1Up2")
		retval = Joy1Up2;
	if(text == "joy1Left2")
		retval = Joy1Left2;
	if(text == "joy1Right2")
		retval = Joy1Right2;
	if(text == "joy1Down2")
		retval = Joy1Down2;
	if(text == "joy1Button1")
		retval = Joy1Button1;
	if(text == "joy1Button2")
		retval = Joy1Button2;
	if(text == "joy1Button3")
		retval = Joy1Button3;
	if(text == "joy1Button4")
		retval = Joy1Button4;
	if(text == "joy1Button5")
		retval = Joy1Button5;
	if(text == "joy1Button6")
		retval = Joy1Button6;
	if(text == "joy1Button7")
		retval = Joy1Button7;
	if(text == "joy1Button8")
		retval = Joy1Button8;
	if(text == "joy1Button9")
		retval = Joy1Button9;
	if(text == "joy1Button10")
		retval = Joy1Button10;
	if(text == "joy2Up")
		retval = Joy2Up;
	if(text == "joy2Left")
		retval = Joy2Left;
	if(text == "joy2Right")
		retval = Joy2Right;
	if(text == "joy2Down")
		retval = Joy2Down;
	if(text == "joy2Trigger1")
		retval = Joy2Trigger1;
	if(text == "joy2Trigger2")
		retval = Joy2Trigger2;
	if(text == "joy2Up2")
		retval = Joy2Up2;
	if(text == "joy2Left2")
		retval = Joy2Left2;
	if(text == "joy2Right2")
		retval = Joy2Right2;
	if(text == "joy2Down2")
		retval = Joy2Down2;
	if(text == "joy2Button1")
		retval = Joy2Button1;
	if(text == "joy2Button2")
		retval = Joy2Button2;
	if(text == "joy2Button3")
		retval = Joy2Button3;
	if(text == "joy2Button4")
		retval = Joy2Button4;
	if(text == "joy2Button5")
		retval = Joy2Button5;
	if(text == "joy2Button6")
		retval = Joy2Button6;
	if(text == "joy2Button7")
		retval = Joy2Button7;
	if(text == "joy2Button8")
		retval = Joy2Button8;
	if(text == "joy2Button9")
		retval = Joy2Button9;
	if(text == "joy2Button10")
		retval = Joy2Button10;
	if(text == "joy3Up")
		retval = Joy3Up;
	if(text == "joy3Left")
		retval = Joy3Left;
	if(text == "joy3Right")
		retval = Joy3Right;
	if(text == "joy3Down")
		retval = Joy3Down;
	if(text == "joy3Trigger1")
		retval = Joy3Trigger1;
	if(text == "joy3Trigger2")
		retval = Joy3Trigger2;
	if(text == "joy3Up2")
		retval = Joy3Up2;
	if(text == "joy3Left2")
		retval = Joy3Left2;
	if(text == "joy3Right2")
		retval = Joy3Right2;
	if(text == "joy3Down2")
		retval = Joy3Down2;
	if(text == "joy3Button1")
		retval = Joy3Button1;
	if(text == "joy3Button2")
		retval = Joy3Button2;
	if(text == "joy3Button3")
		retval = Joy3Button3;
	if(text == "joy3Button4")
		retval = Joy3Button4;
	if(text == "joy3Button5")
		retval = Joy3Button5;
	if(text == "joy3Button6")
		retval = Joy3Button6;
	if(text == "joy3Button7")
		retval = Joy3Button7;
	if(text == "joy3Button8")
		retval = Joy3Button8;
	if(text == "joy3Button9")
		retval = Joy3Button9;
	if(text == "joy3Button10")
		retval = Joy3Button10;

	return retval;
}
