//	OpenGLFont.h - Used to draw truetype fonts.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef OPENGLFONT_H_
#define OPENGLFONT_H_

#include "Drawer.h"
#include "FilePath.h"
#include "OpenGLHeaders.h"

#include "SDL.h"
#include <string>
#include <vector>

//------------------------------------------------------------------------------
///	Used to draw truetype fonts.
class OpenGLFont
{
  public:
	///	Constructor.
	/*!
		\param name The filename of the font (i.e. "georgia.ttf").  Don't give
		the whole path - the class will automatically search the default font
		locations on your system, plus '<your executable path>/fonts'.
		\param maxSize The size to render the font at, in pixels.  This just
		applies to how the font is rendered to its texture atlas; you can scale
		it differently when it comes to rendering it to screen.
	 */
	OpenGLFont(const std::string& name, float maxSize);
	///	Destructor.
	~OpenGLFont();

	///	Draws the text within a space.
	void draw(const std::string& text,
			  float x,
			  float y,
			  float boxWidth,
			  float scale,
			  float r = 1.0f,
			  float g = 1.0f,
			  float b = 1.0f,
			  float a = 1.0f);
	///	For testing.  Just draws the texture atlas.
	void drawAtlas();

	///	Returns true if the font loaded correctly.
	bool getValid() const {return valid;};

	///	Returns the width of a glyph.
	float getGlyphWidth(char c) const;
	///	Returns the height of a glyph.
	float getGlyphHeight(char c) const {return height;};

	///	All the characters that we recognise.
	static const char characters[];
	///	The size of characters.
	static const size_t charactersSize;
  private:
	///	Helper method to look for a font in the various directories it may reside.
	const FilePath getFontPath(const std::string& filename);
	///	Helper method to copy a glyph onto the texture atlas.
	void copyGlyph(uint8_t *atlasData,
				   SDL_Surface *glyph,
				   int x,
				   int y,
				   int yOffset);
	///	Helper method draws a glyph at the appropriate position.
	void drawGlyph(float atlasX,
				   float atlasY,
				   float width,
				   float scale,
				   float x,
				   float y,
				   float r,
				   float g,
				   float b,
				   float a);
	///	Helper method.  Converts a char into the relevant index to glyphs.
	/*!
		\return If it returns characterSize, that's a space.
	 */
	int getGlyphIndex(char c) const
	{
		unsigned int i;

		for(i=0;i<charactersSize;++i)
		{
			if(characters[i] == c)
				break;
		}

		return i;
	};
	///	Helper method to return the next highest number that is a power-of-two.
	/*!
		\return if val is already power-of-two, it will be returned as-is.
	 */
	uint32_t powerOfTwo(uint32_t val);

	///	The font atlas.
	GLuint atlas;

	///	Struct representing a glyph in the atlas.
	struct Glyph
	{
		///	This glyph's character.
		char character;
		///	The glyph's position within the atlas.
		float xPos;
		///	The glyph's position within the atlas.
		float yPos;
		///	The width of the glyph.
		float width;
		///	The starting offset of the glyph.
		float xOffset;
		///	The advance value of the glyph.
		float advance;
	};
	///	The glyphs in the font atlas.
	std::vector<Glyph> glyphs;
	///	The size of the atlas (the atlas is square).
	int atlasSize;
	///	The height of the font.
	float height;

	///	Whether or not the font loaded correctly.
	bool valid;
};

//------------------------------------------------------------------------------
///	Simple singleton used to init and quit SDL_ttf.
class TTFSingleton
{
  public:
	///	Returns the sole instance of the singleton.
	static TTFSingleton& getInstance();

#ifdef WIN32
	///	Destructor.
	~TTFSingleton();
#endif
  private:
	///	Constructor.
	TTFSingleton();
#ifndef WIN32
	///	Destructor.
	~TTFSingleton();
#endif
};

#endif
