//	Letter.cpp - A Letter agent.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Letter.h"
#include "Random.h"
#include "Helpers.h"
#include "GamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "synth/SynthSingleton.h"
#include "synth/SoundHelpers.h"

#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
int Letter::numLetters = 0;

//------------------------------------------------------------------------------
Letter::Letter(const TwoFloats& startPos,
			   char character,
			   const ThreeFloats& col):
Agent(Agent::Jitter),
colour(col),
position(startPos),
jitterCentre(startPos),
jitterCount(4),
wanderSpeed(1.0f),
scale(0.0f),
life(1.0f)
{
	stringstream tempstr;
	int pitch;
	float hue;

	tempstr << character;

	letter = tempstr.str();

	wanderDir.x = (Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f;
	wanderDir.y = (Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f;
	wanderDir = wanderDir.normal();

	while(!Helpers::isValidColour(colour))
	{
		colour.x = Random::getGlobal().getRandomFloat();
		colour.y = Random::getGlobal().getRandomFloat();
		colour.z = Random::getGlobal().getRandomFloat();
	}

	++numLetters;

	hue = SoundHelpers::getHue(colour);
	pitch = (int)(hue * 9.99f);
	SynthSingleton::getInstance().triggerBell(SoundHelpers::getFrequencyForPitch(pitch), 0.5f);
}

//------------------------------------------------------------------------------
Letter::~Letter()
{
	--numLetters;
}

//------------------------------------------------------------------------------
void Letter::draw(Drawer *d, uint32_t font, float alpha)
{
	if(scale < 1.0f)
		scale += 0.01f;

	d->drawText(letter,
				font,
				position.x-12.0f,
				position.y-12.0f,
				128.0f,
				scale,
				colour.x,
				colour.y,
				colour.z,
				life * alpha);
}

//------------------------------------------------------------------------------
void Letter::playerMove(const TwoFloats& pos)
{

}

//------------------------------------------------------------------------------
void Letter::endPlayerMove()
{

}

//------------------------------------------------------------------------------
void Letter::playerAddChar(char character)
{

}

//------------------------------------------------------------------------------
void Letter::nearby(AgentWeak& other, GamePlay *gamePlay)
{
	AgentPtr agent = other.lock();
	AgentPtr following = followTarget.lock();
	MovementState state = getMovementState();

	if(agent)
	{
		if(state == Jitter)
		{
			//In rare cases, Letters will start following other agents.
			if(Random::getGlobal().getRandomFloat() > 0.99f)
			{
				followTarget = other;
				setMovementState(Follow);
			}
		}
		else if(state == Follow)
		{
			//If we're nearby the agent we're Following.
			if((agent == following) && (numLetters < 64) && following)
			{
				if(Random::getGlobal().getRandomFloat() > 0.9975f)
				{
					char newLetter;
					TwoFloats newPos;
					ThreeFloats newCol;
					TwoFloats followPos = following->getPosition();

					//Give birth to a new Letter.
					newPos = position + ((followPos-position)*0.5f);
					newPos.y += 16.0f;
					newLetter = letter[0] + ((following->getLetter()-letter[0])/2);
					if(Random::getGlobal().getRandomFloat() > 0.33f)
						newCol = (colour + ((following->getColour()-colour)*0.5f));
					else
					{
						while(!Helpers::isValidColour(newCol))
						{
							newCol.x = Random::getGlobal().getRandomFloat();
							newCol.y = Random::getGlobal().getRandomFloat();
							newCol.z = Random::getGlobal().getRandomFloat();
						}
					}
					gamePlay->addLetter(newPos, newLetter, newCol);

					//Change movement state.
					jitterCentre = position;
					setMovementState(Wander);
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void Letter::moveJitter()
{
	float hue;
	int pitch;
	float pan;

	--jitterCount;
	if(jitterCount <= 0)
	{
		position.x = jitterCentre.x + ((Random::getGlobal().getRandomFloat() * 8.0f)-4.0f);
		position.y = jitterCentre.y + ((Random::getGlobal().getRandomFloat() * 8.0f)-4.0f);

		prevPos.push_back(position);
		if(prevPos.size() > 128)
			prevPos.erase(prevPos.begin());

		hue = SoundHelpers::getHue(colour);
		pitch = (int)(hue * 9.99f);
		if(position.x < 0.0f)
			pan = 0.0f;
		else if(position.x > 1280.0f)
			pan = 1.0f;
		else
			pan = (position.x/1280.0f);
		SynthSingleton::getInstance().triggerBell(SoundHelpers::getFrequencyForPitch(pitch), pan);

		jitterCount = 4;
	}

	if(life >= 1.0f)
	{
		if(Random::getGlobal().getRandomFloat() > 0.999f)
			life -= 0.01f;
	}
	else
		life -= 0.01f;
}

//------------------------------------------------------------------------------
void Letter::moveWander()
{
	float hue;
	int pitch;
	float pan;

	position += wanderDir * wanderSpeed;

	wanderSpeed += ((Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f);
	if(wanderSpeed < 1.0f)
		wanderSpeed = 1.0f;
	else if(wanderSpeed > 4.0f)
		wanderSpeed = 4.0f;

	if(Random::getGlobal().getRandomFloat() > 0.95f)
	{
		wanderDir.x = (Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f;
		wanderDir.y = (Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f;

		hue = SoundHelpers::getHue(colour);
		pitch = (int)(hue * 9.99f);
		if(position.x < 0.0f)
			pan = 0.0f;
		else if(position.x > 1280.0f)
			pan = 1.0f;
		else
			pan = (position.x/1280.0f);
		SynthSingleton::getInstance().triggerBell(SoundHelpers::getFrequencyForPitch(pitch), pan);
	}

	if(position.x > 1280.0f)
		wanderDir.x = -1.0f;
	else if(position.x < 0.0f)
		wanderDir.x = 1.0f;
	if(position.y > 720.0f)
		wanderDir.y = -1.0f;
	else if(position.y < 0.0f)
		wanderDir.y = 1.0f;

	wanderDir = wanderDir.normal();

	if(Random::getGlobal().getRandomFloat() > 0.999f)
	{
		jitterCentre = position;
		setMovementState(Jitter);
	}

	if(life >= 1.0f)
	{
		if(Random::getGlobal().getRandomFloat() > 0.999f)
			life -= 0.01f;
	}
	else
		life -= 0.01f;
}

//------------------------------------------------------------------------------
void Letter::moveTowards()
{

}

//------------------------------------------------------------------------------
void Letter::moveAway()
{

}

//------------------------------------------------------------------------------
void Letter::moveOrbit()
{

}

//------------------------------------------------------------------------------
void Letter::moveFollow()
{
	AgentPtr agent = followTarget.lock();

	if(agent)
	{
		TwoFloats otherPos = agent->getPosition();

		position += (otherPos - position) * 0.05f;

		//Check we're not at the same position as the agent we're following.
		otherPos = otherPos-position;
		if(otherPos.abs() < 0.5f)
			setMovementState(Wander);

		if(position.x > 1280.0f)
			setMovementState(Wander);
		else if(position.x < 0.0f)
			setMovementState(Wander);
		if(position.y > 720.0f)
			setMovementState(Wander);
		else if(position.y < 0.0f)
			setMovementState(Wander);

		prevPos.push_back(position);
		if(prevPos.size() > 128)
			prevPos.erase(prevPos.begin());

		//On rare occasions, will stop Following at random.
		if(Random::getGlobal().getRandomFloat() > 0.9999f)
		{
			jitterCentre = position;
			setMovementState(Jitter);
		}
	}
	else
	{
		jitterCentre = position;
		setMovementState(Jitter);
	}
}
