//	Letter.h - A Letter agent.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef LETTER_H_
#define LETTER_H_

#include "Agent.h"

///	A Letter agent.
class Letter : public Agent
{
  public:
	///	Constructor.
	Letter(const TwoFloats& startPos, char character, const ThreeFloats& col);
	///	Destructor.
	~Letter();

	///	Draws the agent.
	void draw(Drawer *d, uint32_t font, float alpha);

	///	Called when the player tries to move the agent.
	void playerMove(const TwoFloats& pos);
	///	Called when the player finishes trying to move the agent.
	void endPlayerMove();

	///	Called when the player adds a letter to the agent.
	void playerAddChar(char character);

	///	Called when the agent is near another agent.
	void nearby(AgentWeak& other, GamePlay *gamePlay);

	///	Returns this agent's name/type.
	const std::string getName() const {return "Letter";};
	///	Returns this agent's current position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns a vector of this agent's recent path through the void.
	const std::vector<TwoFloats>& getPath() const {return prevPos;};
	///	Returns this agent's colour.
	const ThreeFloats getColour() const {return colour;};
	///	Returns this agent's letter.
	char getLetter() const {return letter[0];};
	///	Returns this agent's word.
	std::string getWord() const {return letter;};
	///	Returns true if this agent is dead.
	bool getDead() const {return life <= 0.0f;};
  private:
	///	Moves the agent slightly around a static point.
	void moveJitter();
	///	Moves the agent aimlessly.
	void moveWander();
	///	Moves the agent towards a point.
	void moveTowards();
	///	Moves the agent away from a point.
	void moveAway();
	///	Moves the agent in an orbit around a point.
	void moveOrbit();
	///	Moves the agent towards another.
	void moveFollow();

	///	Used to ensure we don't create too many Letters.
	static int numLetters;

	///	The colour of the letter.
	ThreeFloats colour;

	///	The Letter's current position.
	TwoFloats position;
	///	The Letter's previous positions.
	std::vector<TwoFloats> prevPos;

	///	When Jittering, this is the centre point around which the letter jitters.
	TwoFloats jitterCentre;
	///	Used to only jitter every few frames.
	int jitterCount;

	///	When Wandering, this is the direction the letter is currently headed.
	TwoFloats wanderDir;
	///	When Wandering, this is the current speed the letter is moving at.
	float wanderSpeed;

	///	When Following, the letter moves towards this agent's position.
	AgentWeak followTarget;

	///	Which letter this is.
	std::string letter;

	///	Used to scale the letter up when it's born.
	float scale;
	///	Used to kill the letter off after a period.
	float life;
};

#endif
