//	Trail.cpp - A trail agent.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Trail.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "MiscHelper.h"
#include "Random.h"
#include "Helpers.h"
#include "synth/SynthSingleton.h"
#include "synth/SoundHelpers.h"

using namespace std;

//------------------------------------------------------------------------------
GLuint Trail::inkImage = 0;
GLuint Trail::lineImage = 0;
uint32_t Trail::gradientImage = 0;
uint32_t Trail::starburstImages[3] = {0, 0, 0};

//------------------------------------------------------------------------------
Trail::Trail():
Agent(Wander),
starburstIndex(0),
starburstCount(4),
trailIndex(0),
synth(0),
dead(false)
{
	int pitch;
	float hue;

	while(!Helpers::isValidColour(colour))
	{
		colour.x = Random::getGlobal().getRandomFloat();
		colour.y = Random::getGlobal().getRandomFloat();
		colour.z = Random::getGlobal().getRandomFloat();
	}

	starburstRot = Random::getGlobal().getRandomFloat() * 360.0f;
	starburstSpeed = (Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f;

	hue = SoundHelpers::getHue(colour);
	//pitch = (int)((colour.x + colour.y + colour.z) * 3.5f);
	pitch = (int)(hue * 9.99f);
	synth = SynthSingleton::getInstance().activateBrushStrokesSynth(pitch);
}

//------------------------------------------------------------------------------
Trail::~Trail()
{

}

//------------------------------------------------------------------------------
void Trail::draw(Drawer *d, uint32_t font, float alpha)
{
	int i;
	float pan;
	float reduceSize;
	float reduceSize2;
	TwoFloats tempPos;
	float alpha1, alpha2;
	const float trailWidth = 26.0f;
	int trailSize = (int)trailPos.size();

	if(!inkImage)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::ink2_png, Images::ink2_pngSize);

		glGenTextures(1, &inkImage);

		glBindTexture(GL_TEXTURE_2D, inkImage);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 data.width,
					 data.height,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 data.data);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		glBindTexture(GL_TEXTURE_2D, 0);

		data = imLoad.loadImage(Images::line_png, Images::line_pngSize);

		glGenTextures(1, &lineImage);

		glBindTexture(GL_TEXTURE_2D, lineImage);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 data.width,
					 data.height,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 data.data);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		glBindTexture(GL_TEXTURE_2D, 0);

		data = imLoad.loadImage(Images::radialgradient_png, Images::radialgradient_pngSize);
		gradientImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::starburst0_png, Images::starburst0_pngSize);
		starburstImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::starburst1_png, Images::starburst1_pngSize);
		starburstImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::starburst2_png, Images::starburst2_pngSize);
		starburstImages[2] = d->loadImage(data.data, data.width, data.height);
	}

	//Update drawn trail.
	for(i=(TrailSize-1);i>0;--i)
		trailDraw[i] = trailDraw[i-1];

	tempPos = trailPos[trailIndex] + startPos;
	if(trailDraw[0] != tempPos)
	{
		trailDraw[0] += (tempPos-trailDraw[0]) * 0.1f;
		if((tempPos-trailDraw[0]).abs() < 0.1f)
			trailDraw[0] = tempPos;
	}
	else
	{
		trailDraw[0] = tempPos;
	}

	//Update the synth.
	if(synth->getActive())
	{
		synth->setTables(trailDraw);

		if(tempPos.x < 0.0f)
			pan = 0.0f;
		else if(tempPos.x > 1280.0f)
			pan = 1.0f;
		else pan = (tempPos.x/1280.0f);
		synth->setPan(pan);
	}

	//Draw trail.
	glBindTexture(GL_TEXTURE_2D, inkImage);

	//The two intersecting lines where the line we're drawing meets its
	//neighbours.
	TwoFloats intersections[2];

	intersections[1] = (trailDraw[1] - trailDraw[0]).perpendicular().normal();
	for(i=1;i<(TrailSize-2);++i)
	{
		if(i < 2)
		{
			//The 2 alpha values.
			alpha1 = ((float)i/3.0f);
			alpha2 = ((float)(i+1)/3.0f);
		}
		else
		{
			alpha1 = 1.0f-((float)i/(float)TrailSize);
			alpha2 = 1.0f-((float)(i+1)/(float)TrailSize);
		}

		reduceSize = fakeLog(alpha1);
		reduceSize2 = fakeLog(alpha2);

		//The 4 trail points we need to calculate the correct corners for the
		//line trailPos[i] -> trailPos[i+1].
		TwoFloats a0(trailDraw[i-1]);
		TwoFloats a1(trailDraw[i]);
		TwoFloats a2(trailDraw[i+1]);
		TwoFloats a3(trailDraw[i+2]);

		//The 4 corners of the quad we use to draw the line
		//trailPos[i] -> trailPos[i+1]. The arrangement is:
		//3--2
		//|..|
		//|..|
		//0--1
		TwoFloats corners[4];

		//Start with the 'bottom-left' corner.
		intersections[0] = intersections[1];
		corners[0] = a1 - (intersections[0] * trailWidth * reduceSize);

		//Now the 'top-left' corner.
		corners[3] = a1 + (intersections[0] * trailWidth * reduceSize);

		//Now the 'bottom-right' corner.
		intersections[1] = (a3-a2).perpendicular().normal();
		corners[1] = a2 - (intersections[1] * trailWidth * reduceSize2);

		//Now the 'top-left' corner.
		corners[2] = a2 + (intersections[1] * trailWidth * reduceSize2);

		//Now draw the line.
		//glColor4f(1.0f, 1.0f, 1.0f, alpha);
		if(fabsf(intersections[0].getAngle(intersections[1])) < (2.0f * 1.570796f))
		{
			glBegin(GL_QUADS);
			{
				glColor4f(colour.x, colour.y, colour.z, alpha1*alpha);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glVertex2f(corners[0].x, corners[0].y);
				glColor4f(colour.x, colour.y, colour.z, alpha2*alpha);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glVertex2f(corners[1].x, corners[1].y);
				glColor4f(colour.x, colour.y, colour.z, alpha2*alpha);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glVertex2f(corners[2].x, corners[2].y);
				glColor4f(colour.x, colour.y, colour.z, alpha1*alpha);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glVertex2f(corners[3].x, corners[3].y);
			}
			glEnd();
		}
	}
	d->drawImage(gradientImage,
				 trailDraw[1].x-(16.0f * 1.1f),
				 trailDraw[1].y-(16.0f * 1.1f),
				 1.1f,
				 1.1f,
				 colour.x,
				 colour.y,
				 colour.z,
				 alpha);
	d->drawImageRot(starburstImages[starburstIndex],
					trailDraw[1].x-(32.0f * 1.1f),
					trailDraw[1].y-(32.0f * 1.1f),
					starburstRot,
					32.0f,
					32.0f,
					1.1f,
					1.1f,
					colour.x + 0.2f,
					colour.y + 0.2f,
					colour.z + 0.2f,
					0.33f * alpha);

	--starburstCount;
	if(starburstCount < 0)
	{
		++starburstIndex;
		starburstIndex %= 3;

		starburstCount = 4;
	}

	starburstRot += starburstSpeed;
	if(starburstRot > 360.0f)
		starburstRot -= 360.0f;
	else if(starburstRot < 0.0f)
		starburstRot += 360.0f;

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBindTexture(GL_TEXTURE_2D, 0);
}

//------------------------------------------------------------------------------
void Trail::playerMove(const TwoFloats& point)
{
	int i;
	TwoFloats tempPos;

	if(trailPos.size() < 1)
	{
		/*TwoFloats tempPos = point;

		tempPos.x -= (116.0f/2.0f)/2.0f;
		trailPos.push_back(tempPos);*/
		startPos = point;

		for(i=0;i<TrailSize;++i)
			trailDraw[i] = point;
	}
	else if(trailPos.size() >= 96)
		return;

	tempPos = point;
	trailPos.push_back(tempPos-startPos);

	//Add interpolated positions back to the start.
	if(trailPos.size() >= 96)
	{
		TwoFloats tempDir;
		TwoFloats lastPos[4];
		TwoFloats dir[2];
		
		lastPos[0] = trailPos[trailPos.size()-2];
		lastPos[1] = trailPos[trailPos.size()-1];
		lastPos[2] = trailPos[0];
		lastPos[3] = trailPos[1];

		dir[0] = (lastPos[1]-lastPos[0]).normal();
		dir[1] = (lastPos[3]-lastPos[2]).normal();

		tempPos = lastPos[1];
		for(i=0;i<32;++i)
		{
			tempDir = (dir[0] * (1.0f-((float)i/31.0f))) + (dir[1] * ((float)i/31.0f));
			tempPos += tempDir * 4.0f;
			if(i == 31)
				tempDir = tempDir;

			trailPos.push_back(tempPos);
		}
	}
}

//------------------------------------------------------------------------------
void Trail::endPlayerMove()
{
	int i;
	TwoFloats tempPos;

	//In case we accidentally get a Trail that's way too small.
	if(trailPos.size() < 5)
	{
		dead = true;
		return;
	}

	//Add interpolated positions back to the start.
	TwoFloats tempDir;
	TwoFloats lastPos[4];
	TwoFloats dir[2];
		
	lastPos[0] = trailPos[trailPos.size()-2];
	lastPos[1] = trailPos[trailPos.size()-1];
	lastPos[2] = trailPos[0];
	lastPos[3] = trailPos[1];

	dir[0] = (lastPos[1]-lastPos[0]).normal();
	dir[1] = (lastPos[3]-lastPos[2]).normal();

	tempPos = lastPos[1];
	for(i=0;i<32;++i)
	{
		tempDir = (dir[0] * (1.0f-((float)i/31.0f))) + (dir[1] * ((float)i/31.0f));
		tempPos += tempDir * 4.0f;
		if(i == 31)
			tempDir = tempDir;

		trailPos.push_back(tempPos);
	}
}

//------------------------------------------------------------------------------
void Trail::playerAddChar(char character)
{

}

//------------------------------------------------------------------------------
void Trail::nearby(AgentWeak& other, GamePlay *gamePlay)
{
	AgentPtr agent = other.lock();

	if(agent)
	{
		if(agent->getName() == "Trail")
		{
			//In rare cases, Trails will assume the path of nearby agents.
			if(Random::getGlobal().getRandomFloat() > 0.9975f)
			{
				const vector<TwoFloats>& otherPath = agent->getPath();

				if(otherPath.size() > 0)
				{
					trailPos.clear();
					trailPos = otherPath;
					if(trailIndex >= (int)trailPos.size())
						trailIndex = 0;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void Trail::moveJitter()
{

}

//------------------------------------------------------------------------------
void Trail::moveWander()
{
	int i;

	++trailIndex;
	if(trailIndex >= (int)trailPos.size())
	{
		bool reset = false;

		trailIndex = 0;
		startPos = trailDraw[1];

		//Make sure we're always on screen.
		if(startPos.x < 0.0f)
		{
			//startPos.x = 1280.0f + 64.0f;
			reset = true;
		}
		else if(startPos.x > (1280.0f+0.0f))
		{
			//startPos.x = -64.0f;
			reset = true;
		}
		if(reset)
		{
			for(i=0;i<(int)trailPos.size();++i)
				trailPos[i].x = -trailPos[i].x;
			reset = false;
		}

		if(startPos.y < 0.0f)
		{
			//startPos.y = 720.0f + 64.0f;
			reset = true;
		}
		else if(startPos.y > (720.0f+0.0f))
		{
			//startPos.y = -64.0f;
			reset = true;
		}
		if(reset)
		{
			for(i=0;i<(int)trailPos.size();++i)
				trailPos[i].y = -trailPos[i].y;
			reset = false;
		}

		/*if(reset)
		{
			for(i=0;i<TrailSize;++i)
				trailDraw[i] = startPos;
		}*/
	}
}

//------------------------------------------------------------------------------
void Trail::moveTowards()
{

}

//------------------------------------------------------------------------------
void Trail::moveAway()
{

}

//------------------------------------------------------------------------------
void Trail::moveOrbit()
{

}

//------------------------------------------------------------------------------
void Trail::moveFollow()
{

}
