//	Trail.h - A trail agent.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TRAIL_H_
#define TRAIL_H_

#include "../Agent.h"
#include "NiallsDrawingLib/OpenGLHeaders.h"

#include <vector>

class BrushStrokesSynth;

///	A trail agent.
class Trail : public Agent
{
  public:
	///	Constructor.
	Trail();
	///	Destructor.
	~Trail();

	///	Draws the agent.
	void draw(Drawer *d, uint32_t font, float alpha);

	///	Adds a point to the trail.
	void playerMove(const TwoFloats& point);
	///	Called when the player finishes trying to move the agent.
	void endPlayerMove();

	///	Called when the player adds a letter to the agent.
	void playerAddChar(char character);

	///	Called when the agent is near another agent.
	void nearby(AgentWeak& other, GamePlay *gamePlay);

	///	Returns this agent's name/type.
	const std::string getName() const {return "Trail";};
	///	Returns this agent's current position.
	const TwoFloats& getPosition() const {return trailDraw[1];};
	///	Returns a vector of this agent's recent path through the void.
	const std::vector<TwoFloats>& getPath() const {return trailPos;};
	///	Returns this agent's colour.
	const ThreeFloats getColour() const {return colour;};
	///	Returns this agent's letter.
	char getLetter() const {return 't';};
	///	Returns this agent's word.
	std::string getWord() const {return "t";};
	///	Returns true if this agent is dead.
	bool getDead() const {return dead;};
  private:
	///	Moves the agent slightly around a static point.
	void moveJitter();
	///	Moves the agent aimlessly.
	void moveWander();
	///	Moves the agent towards a point.
	void moveTowards();
	///	Moves the agent away from a point.
	void moveAway();
	///	Moves the agent in an orbit around a point.
	void moveOrbit();
	///	Moves the agent towards another.
	void moveFollow();

	///	B-spline interpolation helper method.
	inline float bSplineInterp(float x0,
							   float x1,
							   float x2,
							   float x3,
							   float val)
{
		// 4-point, 3rd-order B-spline (x-form)
		float ym1py1 = x0+x2;
		float c0 = (1.0f/6.0f) * ym1py1 + (2.0f/3.0f) * x1;
		float c1 = (1.0f/2.0f) * (x2-x0);
		float c2 = (1.0f/2.0f) * ym1py1 - x1;
		float c3 = (1.0f/2.0f) * (x1-x2) + (1.0f/6.0f) * (x3-x0);
		return ((c3*val+c2)*val+c1)*val+c0;
	};

	///	The ink image used to draw the trail.
	static GLuint inkImage;
	///	The (hopefully temporary) image used to draw the trail.
	static GLuint lineImage;
	///	The radial gradient image.
	static uint32_t gradientImage;
	///	The 3 starburst images.
	static uint32_t starburstImages[3];

	///	The size of the drawn trail.
	enum
	{
		TrailSize = 48
	};

	///	The colour of the trail.
	ThreeFloats colour;
	///	Which of the 3 starburst images we're currently drawing.
	int starburstIndex;
	///	Used to switch through the 3 starburst images.
	int starburstCount;
	///	Used to rotate the starburst.
	float starburstRot;
	///	Used to rotate the starburst.
	float starburstSpeed;

	///	Current point w/in trailPos.
	int trailIndex;
	///	The current trail to follow.
	std::vector<TwoFloats> trailPos;
	///	The trail we actually draw.
	TwoFloats trailDraw[TrailSize];

	///	The current start point (trailDraw is then calculated relative to this).
	TwoFloats startPos;

	///	Our synth.
	BrushStrokesSynth *synth;

	///	True if the Trail should be killed off.
	bool dead;
};

#endif
