//	Word.cpp - A Word agent.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Word.h"
#include "NiallsDrawingLib/Drawer.h"
#include "Helpers.h"
#include "Random.h"
#include "GamePlay.h"
#include "synth/SynthSingleton.h"
#include "synth/SoundHelpers.h"

using namespace std;

//------------------------------------------------------------------------------
int Word::numWords = 0;

//------------------------------------------------------------------------------
Word::Word(const TwoFloats& startPos, std::string text, const ThreeFloats& col):
Agent(Jitter),
colour(col),
position(startPos),
word(text),
wanderInterp(0.0f),
spaceScale(1.0f),
killLetters(30),
synthLetter(0),
synthCount(0),
synthScale(1.0f)
{
	int i;
	/*int pitch;
	float hue;*/

	while(!Helpers::isValidColour(colour))
	{
		colour.x = Random::getGlobal().getRandomFloat();
		colour.y = Random::getGlobal().getRandomFloat();
		colour.z = Random::getGlobal().getRandomFloat();
	}

	for(i=0;i<ColourSize;++i)
		colourBuffer[i] = colour;

	for(i=0;i<(int)word.length();++i)
	{
		characters.push_back(Character());
		characters[i].position.x = 0.0f;
		characters[i].position.y = Random::getGlobal().getRandomFloat() * 1.0f;
		characters[i].ySpeed = 0.01f + (Random::getGlobal().getRandomFloat() * 0.05f);
		characters[i].colour = colour;
		characters[i].alpha = 1.0f;
	}

	life = 1200 + (int)(Random::getGlobal().getRandomFloat() * 1200);

	++numWords;

	/*hue = SoundHelpers::getHue(colour);
	pitch = (int)(hue * 9.99f);
	SynthSingleton::getInstance().triggerWord(SoundHelpers::getFrequencyForPitch(pitch), 0.5f);*/

	synthTime = (int)(Random::getGlobal().getRandomFloat() * 20) + 30;
}

//------------------------------------------------------------------------------
Word::~Word()
{
	--numWords;
}

//------------------------------------------------------------------------------
void Word::draw(Drawer *d, uint32_t font, float alpha)
{
	int i;
	float y;
	float scale;
	float x = 0.0f;
	int bufIndex = 0;

	for(i=0;i<(int)word.length();++i)
	{
		y = sinf(characters[i].position.y * 6.283185f) * 4.0f;
		characters[i].position.y += characters[i].ySpeed;
		if(characters[i].position.y > 1.0f)
			characters[i].position.y = 0.0f;

		if(i < (int)prevPos.size())
			y += prevPos[i].y;
		else
			y += position.y;

		bufIndex = (int)(((float)i/(float)word.length()) * (float)ColourSize);
		characters[i].colour = colourBuffer[bufIndex];

		if(i == synthLetter)
			scale = synthScale;
		else
			scale = 1.0f;

		d->drawText(word.substr(i, 1),
					font,
					(x /* spaceScale*/) + position.x,
					y - (scale * 4.0f),
					64.0f,
					scale,
					characters[i].colour.x,
					characters[i].colour.y,
					characters[i].colour.z,
					characters[i].alpha * alpha);

		if(characters[i].position.x == 0.0f)
		{
			characters[i].width = d->getGlyphWidth(font, word[i]);
			characters[i].position.x = x;
			x += characters[i].width;
		}
		else
			x += characters[i].width;
	}

	if(synthScale > 1.0f)
		synthScale -= 0.05f;

	for(i=(ColourSize-1);i>0;--i)
		colourBuffer[i] = colourBuffer[i-1];
}

//------------------------------------------------------------------------------
void Word::playerMove(const TwoFloats& pos)
{

}

//------------------------------------------------------------------------------
void Word::endPlayerMove()
{

}

//------------------------------------------------------------------------------
void Word::playerAddChar(char character)
{
	word.append(1, character);

	characters.push_back(Character());
	characters[characters.size()-1].position.x = 0.0f;
	characters[characters.size()-1].position.y = Random::getGlobal().getRandomFloat() * 1.0f;
	characters[characters.size()-1].ySpeed = 0.01f + (Random::getGlobal().getRandomFloat() * 0.05f);
	characters[characters.size()-1].colour = colour;
	characters[characters.size()-1].alpha = 1.0f;
}

//------------------------------------------------------------------------------
void Word::nearby(AgentWeak& other, GamePlay *gamePlay)
{
	AgentPtr agent = other.lock();
	AgentPtr following = followTarget.lock();
	MovementState state = getMovementState();

	if(agent)
	{
		if(state == Jitter)
		{
			//In rare cases, Words will start following other agents.
			if(Random::getGlobal().getRandomFloat() > 0.999f)
			{
				followTarget = other;
				setMovementState(Follow);
			}
		}
		else if(state == Follow)
		{
			//If we're nearby another Word or Letter.
			if(agent->getName() == "Word")
			{
				//Give birth to a new Word.
				if((Random::getGlobal().getRandomFloat() > 0.99f) && (numWords < 32) && following)
				{
					int length;
					string newWord;
					TwoFloats newPos;
					ThreeFloats newCol;
					TwoFloats followPos = following->getPosition();

					//Give birth to a new Letter.
					newPos = position + ((followPos-position)*0.5f);
					newPos.y += 16.0f;
					//Make a word out of a combination of the two parents.
					if(Random::getGlobal().getRandomFloat() > 0.5f)
					{
						newWord = word;
						newWord.append(following->getWord());
						length = 4 + (int)(Random::getGlobal().getRandomFloat() * 14.0f);
						if((int)newWord.length() > length)
							newWord = newWord.substr((newWord.length()-length)/2, length);
					}
					//Take one of the parent's words.
					else if(Random::getGlobal().getRandomFloat() > 0.5f)
						newWord = word;
					else //Ditto.
						newWord = following->getWord();
					newCol = (colour + ((following->getColour()-colour)*0.5f));
					gamePlay->addWord(newPos, newWord, newCol);

					//Change movement state.
					setMovementState(Wander);

					//Reduce life.
					life -= 400;
				}
			}
		}

		//In not-so-rare cases, Words will take on other agents' colours.
		if(Random::getGlobal().getRandomFloat() > 0.66f)
			colourBuffer[0] = agent->getColour();
	}
}

//------------------------------------------------------------------------------
void Word::moveJitter()
{
	//Occasionally start Wandering.
	if(Random::getGlobal().getRandomFloat() > 0.9999f)
		setMovementState(Wander);

	if(spaceScale > 1.0f)
	{
		spaceScale -= 0.01f;
		if(spaceScale < 1.0f)
			spaceScale = 1.0f;
	}
	else if(spaceScale < 1.0f)
	{
		spaceScale += 0.001f;
		if(spaceScale > 1.0f)
			spaceScale = 1.0f;
	}

	prevPos.push_back(position);
	if(prevPos.size() > 128)
		prevPos.erase(prevPos.begin());

	updateLife();
}

//------------------------------------------------------------------------------
void Word::moveWander()
{
	TwoFloats temp2f;
	TwoFloats oldPos = position;

	//Generate a couple of random directions.
	if(wanderInterp == 0.0f)
	{
		wanderDir[0].x = (Random::getGlobal().getRandomFloat()*2.0f)-1.0f;
		wanderDir[0].y = (Random::getGlobal().getRandomFloat()*2.0f)-1.0f;
		wanderDir[0] = wanderDir[0].normal();

		wanderDir[1].x = (Random::getGlobal().getRandomFloat()*2.0f)-1.0f;
		wanderDir[1].y = (Random::getGlobal().getRandomFloat()*2.0f)-1.0f;
		wanderDir[1] = wanderDir[1].normal();
	}

	temp2f = (wanderDir[0] * (1.0f-wanderInterp)) + (wanderDir[1] * wanderInterp);
	position += temp2f * 3.0f;

	prevPos.push_back(position);
	if(prevPos.size() > 128)
		prevPos.erase(prevPos.begin());

	//Make sure we don't wander offscreen.
	if(position.x > 1280.0f)
	{
		wanderInterp = 0.0f;
		wanderDir[1].x = -1.0f;
		//wanderDir[1].y = 0.0f;
		wanderDir[0] = wanderDir[1];
	}
	else if(position.x < 0.0f)
	{
		wanderInterp = 0.0f;
		wanderDir[1].x = 1.0f;
		//wanderDir[1].y = 0.0f;
		wanderDir[0] = wanderDir[1];
	}
	if(position.y > 720.0f)
	{
		wanderInterp = 0.0f;
		//wanderDir[1].x = 0.0f;
		wanderDir[1].y = -1.0f;
		wanderDir[0] = wanderDir[1];
	}
	else if(position.y < 0.0f)
	{
		wanderInterp = 0.0f;
		//wanderDir[1].x = 0.0f;
		wanderDir[1].y = 1.0f;
		wanderDir[0] = wanderDir[1];
	}
	wanderDir[0] = wanderDir[0].normal();
	wanderDir[1] = wanderDir[1].normal();

	updateSpaceScale(oldPos);

	wanderInterp += 0.005f;
	if(wanderInterp > 1.0f)
	{
		wanderInterp -= 1.0f;

		wanderDir[0] = wanderDir[1];

		wanderDir[1].x = (Random::getGlobal().getRandomFloat()*2.0f)-1.0f;
		wanderDir[1].y = (Random::getGlobal().getRandomFloat()*2.0f)-1.0f;
		wanderDir[1] = wanderDir[1].normal();
	}

	if(Random::getGlobal().getRandomFloat() > 0.999f)
	{
		wanderInterp = 0.0f;
		setMovementState(Jitter);
	}

	updateLife();
}

//------------------------------------------------------------------------------
void Word::moveTowards()
{

	updateLife();
}

//------------------------------------------------------------------------------
void Word::moveAway()
{

	updateLife();
}

//------------------------------------------------------------------------------
void Word::moveOrbit()
{

	updateLife();
}

//------------------------------------------------------------------------------
void Word::moveFollow()
{
	AgentPtr agent = followTarget.lock();

	if(agent)
	{
		TwoFloats oldPos = position;
		TwoFloats otherPos = agent->getPosition();

		position += (otherPos - position) * 0.05f;

		updateSpaceScale(oldPos);

		//Check we're not at the same position as the agent we're following.
		otherPos = otherPos-position;
		if(otherPos.abs() < 0.5f)
			setMovementState(Wander);

		if(position.x > 1280.0f)
			setMovementState(Wander);
		else if(position.x < 0.0f)
			setMovementState(Wander);
		if(position.y > 720.0f)
			setMovementState(Wander);
		else if(position.y < 0.0f)
			setMovementState(Wander);

		prevPos.push_back(position);
		if(prevPos.size() > 128)
			prevPos.erase(prevPos.begin());

		//On rare occasions, will stop Following at random.
		if(Random::getGlobal().getRandomFloat() > 0.9999f)
			setMovementState(Jitter);
	}
	else
		setMovementState(Jitter);

	updateLife();
}

//------------------------------------------------------------------------------
void Word::updateLife()
{
	int i;
	int pitch;
	float hue;
	float pan;

	if(life > 0)
	{
		--life;
		if(!life)
		{
			killLetters = 30;
			characters[characters.size()-1].alpha = 0.99f;
		}
	}
	else
	{
		//Update characters' alphas, remove characters if they're dead.
		for(i=(word.length()-1);i>=0;--i)
		{
			if(characters[i].alpha < 1.0f)
			{
				characters[i].alpha -= 0.01f;
				if(characters[i].alpha <= 0.0f)
					word = word.substr(0, i);
			}
		}

		//Kill new letters.
		--killLetters;
		if(!killLetters)
		{
			killLetters = 30;

			for(i=(word.length()-1);i>=0;--i)
			{
				if(characters[i].alpha >= 1.0f)
				{
					characters[i].alpha = 0.99f;
					break;
				}
			}
		}
	}

	//Tick through letters, triggering a WordSynth when we hit the next one.
	--synthCount;
	if(synthCount <= 0)
	{
		++synthLetter;
		synthLetter %= word.length();
		if(synthLetter == (word.length()-1))
			synthCount = synthTime * 4;
		else
			synthCount = synthTime;
		synthScale = 2.0f;

		hue = SoundHelpers::getHue(characters[synthLetter].colour);
		pitch = getPitchFromLetter(word[synthLetter]);
		if(position.x < 0.0f)
			pan = 0.0f;
		else if(position.x > 1280.0f)
			pan = 1.0f;
		else
			pan = (position.x/1280.0f);
		SynthSingleton::getInstance().triggerWord(SoundHelpers::getFrequencyForPitch(pitch), hue, pan);
	}
}

//------------------------------------------------------------------------------
void Word::updateSpaceScale(const TwoFloats& oldPos)
{
	if((position.x < oldPos.x) && (spaceScale < 2.0f))
		spaceScale += 0.01f;
	else if((position.x > oldPos.x) && (spaceScale > 0.5f))
		spaceScale -= 0.001f;
	else if(position.x = oldPos.x)
	{
		if(spaceScale > 1.0f)
		{
			spaceScale -= 0.01f;
			if(spaceScale < 1.0f)
				spaceScale = 1.0f;
		}
		else if(spaceScale < 1.0f)
		{
			spaceScale += 0.001f;
			if(spaceScale > 1.0f)
				spaceScale = 1.0f;
		}
	}
}
