//	Word.h - A Word agent.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef WORD_H_
#define WORD_H_

#include "Agent.h"
#include "synth/SoundHelpers.h"

#include <string>
#include <vector>

///	A Word agent.
class Word : public Agent
{
  public:
	///	Constructor.
	Word(const TwoFloats& startPos, std::string text, const ThreeFloats& col);
	///	Destructor.
	~Word();

	///	Draws the agent.
	void draw(Drawer *d, uint32_t font, float alpha);

	///	Called when the player tries to move the agent.
	void playerMove(const TwoFloats& pos);
	///	Called when the player finishes trying to move the agent.
	void endPlayerMove();

	///	Called when the player adds a letter to the agent.
	void playerAddChar(char character);

	///	Called when the agent is near another agent.
	void nearby(AgentWeak& other, GamePlay *gamePlay);

	///	Returns this agent's name/type.
	const std::string getName() const {return "Word";};
	///	Returns this agent's current position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns a vector of this agent's recent path through the void.
	const std::vector<TwoFloats>& getPath() const {return prevPos;};
	///	Returns this agent's colour.
	const ThreeFloats getColour() const {return colour;};
	///	Returns this agent's letter.
	char getLetter() const {return word[0];};
	///	Returns this agent's word.
	std::string getWord() const {return word;};
	///	Returns true if this agent is dead.
	bool getDead() const {return (word.length() <= 1);};
  private:
	///	Moves the agent slightly around a static point.
	void moveJitter();
	///	Moves the agent aimlessly.
	void moveWander();
	///	Moves the agent towards a point.
	void moveTowards();
	///	Moves the agent away from a point.
	void moveAway();
	///	Moves the agent in an orbit around a point.
	void moveOrbit();
	///	Moves the agent towards another.
	void moveFollow();

	///	Helper method to update the word's life from the various move... methods.
	void updateLife();

	///	Helper method to update spaceScale correctly.
	void updateSpaceScale(const TwoFloats& oldPos);

	///	Helper method: converts a letter into a pitch index.
	inline int getPitchFromLetter(char c)
	{
		return ((int)c - 96) % SoundHelpers::NumPitches;
	};

	///	Struct representing a single character in the word.
	struct Character
	{
		///	The character's position (relative to the word).
		TwoFloats position;
		///	The character's width.
		float width;
		///	Used to bounce the character in the y-axis.
		float ySpeed;
		///	This character's colour.
		ThreeFloats colour;
		///	This character's alpha.
		float alpha;
	};

	///	The size of the colour buffer.
	enum
	{
		ColourSize = 128
	};

	///	Used to ensure we don't create too many Words.
	static int numWords;

	///	The colour of the word.
	ThreeFloats colour;
	///	The colour buffer used to interpolate between colours.
	ThreeFloats colourBuffer[ColourSize];

	///	The word's current position.
	TwoFloats position;
	///	The word's previous positions.
	std::vector<TwoFloats> prevPos;

	///	The word itself.
	std::string word;
	///	The characters in the word.
	std::vector<Character> characters;

	///	When Wandering, these are the 2 directions we're interpolating between.
	TwoFloats wanderDir[2];
	///	When Wandering, this is the interpolation value used in wanderDir.
	float wanderInterp;

	///	When Following, the letter moves towards this agent's position.
	AgentWeak followTarget;
	///	Used to space out or squeeze together the characters when the word's moving horizontally.
	float spaceScale;

	///	Used to kill the word off after a while.
	int life;
	///	Used to kill off the word's letters one-by-one.
	int killLetters;

	///	Used to tick through the letters, triggering WordSynths as we do.
	int synthLetter;
	///	Used to tick through the letters, triggering WordSynths as we do.
	int synthCount;
	///	Used to tick through the letters, triggering WordSynths as we do.
	int synthTime;
	///	Scales a letter when it's triggered.
	float synthScale;
};

#endif
