//	BellSynth.cpp - Simple bell synth based on Risset's table of partials.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BellSynth.h"
#include "MiscHelper.h"
#include "SoundHelpers.h"
#include "Random.h"

#include <cmath>

//------------------------------------------------------------------------------
BellSynth::BellSynth():
frequency(SoundHelpers::getFrequencyForPitch(0)),
pan(0.5f),
active(false)
{
	float dur = 0.2f; //0.2 second.
	float amp = 1.0f;

	//Randomise dur.
	dur += Random::getGlobal().getRandomFloat() * 0.1f;

	partials[0].duration = dur;
	partials[1].duration = dur * 0.9f;
	partials[2].duration = dur * 0.65f;
	partials[3].duration = dur * 0.55f;
	partials[4].duration = dur * 0.325f;
	partials[5].duration = dur * 0.35f;
	partials[6].duration = dur * 0.25f;
	partials[7].duration = dur * 0.2f;
	partials[8].duration = dur * 0.15f;
	partials[9].duration = dur * 0.1f;
	partials[10].duration = dur * 0.75f;

	partials[0].amplitude = amp;
	partials[1].amplitude = amp * 0.67f;
	partials[2].amplitude = amp;
	partials[3].amplitude = amp * 1.8f;
	partials[4].amplitude = amp * 2.67f;
	partials[5].amplitude = amp * 1.67f;
	partials[6].amplitude = amp * 1.46f;
	partials[7].amplitude = amp * 1.33f;
	partials[8].amplitude = amp * 1.33f;
	partials[9].amplitude = amp;
	partials[10].amplitude = amp * 1.33f;
}

//------------------------------------------------------------------------------
BellSynth::~BellSynth()
{

}

//------------------------------------------------------------------------------
void BellSynth::getAudio(float *buffer, int numSamples)
{
	int i, j;
	float amp;
	float tempf;
	float midPoint;
	float midPointMult;

	for(i=0;i<numSamples;++i)
	{
		tempf = 0.0f;

		for(j=0;j<NumPartials;++j)
		{
			if(partials[j].ampEnv < partials[j].duration)
			{
				partials[j].x0 -= (partials[j].coeff * partials[j].x1);
				partials[j].x1 += (partials[j].coeff * partials[j].x0);

				midPoint = partials[j].duration * 0.05f;
				midPointMult = (1.0f/(partials[j].duration - midPoint));

				if(partials[j].release)
					amp = fakeLog(1.0f - ((partials[j].ampEnv - midPoint) * (1.0f/(partials[j].duration - midPoint))));
				else 
					amp = fakeLog(partials[j].ampEnv * (1.0f/midPoint));

				tempf += partials[j].x1 * partials[j].amplitude * amp * 0.005f;

				partials[j].ampEnv += (1.0f/44100.0f);
				if(partials[j].ampEnv >= midPoint)
					partials[j].release = true;
			}
		}

		tempf *= amplitude;

		buffer[(i*2)] += tempf * (1.0f-pan);
		buffer[(i*2)+1] += tempf * pan;

		if(partials[0].ampEnv >= partials[0].duration)
			active = false;
	}
}

//------------------------------------------------------------------------------
void BellSynth::trigger()
{
	int i;

	for(i=0;i<NumPartials;++i)
	{
		partials[i].x0 = 1.0f;
		partials[i].x1 = 0.0f;

		partials[i].ampEnv = 0.0f;
		partials[i].release = false;
	}
	amplitude = 0.5f + (0.5f * Random::getGlobal().getRandomFloat());

	active = true;
}

//------------------------------------------------------------------------------
void BellSynth::setFrequency(float val)
{
	int i;

	frequency = val * 2.0f;

	frequency += (Random::getGlobal().getRandomFloat() * 4.0f)-2.0f;

	partials[0].frequency = frequency * 0.56f;
	partials[1].frequency = (frequency * 0.56f) + 1.0f;
	partials[2].frequency = frequency * 0.92f;
	partials[3].frequency = (frequency * 0.92f) + 1.7f;
	partials[4].frequency = frequency * 1.19f;
	partials[5].frequency = frequency * 1.7f;
	partials[6].frequency = frequency * 2.0f;
	partials[7].frequency = frequency * 2.74f;
	partials[8].frequency = frequency * 3.0f;
	partials[9].frequency = frequency * 3.76f;
	partials[10].frequency = frequency * 4.07f;

	for(i=0;i<NumPartials;++i)
	{
		partials[i].x0 = 1.0f;
		partials[i].x1 = 0.0f;
		partials[i].coeff = 2.0f*sinf(3.1415926535897932384626433832795f*partials[i].frequency*2.0f/44100.0f);
	}
}

//------------------------------------------------------------------------------
void BellSynth::setPan(float val)
{
	pan = val;
}
