//	BrushStrokesSynth.cpp - Synth class based on my Brush Strokes VST plugin.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BrushStrokesSynth.h"
#include "SoundHelpers.h"

//------------------------------------------------------------------------------
BrushStrokesSynth::BrushStrokesSynth():
active(false),
fadeOut(1.0f),
fadeIn(0.0f),
tableInterp(0.0f),
position(0.0f),
pan(0.5f)
{
	setFrequency(SoundHelpers::getFrequencyForPitch(0));
}

//------------------------------------------------------------------------------
BrushStrokesSynth::~BrushStrokesSynth()
{

}

//------------------------------------------------------------------------------
void BrushStrokesSynth::getAudio(float *buffer, int numSamples)
{
	int i, j;
	int pos[4];
	float tempf;
	TwoFloats outVal;
	float tableVals[4];

	for(i=0;i<numSamples;++i)
	{
		//The 4 table indices used in our bSpline interpolation.
		pos[2] = (int)position+2;
		pos[0] = pos[2]-2;
		pos[1] = pos[2]-1;
		pos[3] = pos[2]+1;

		//The 4 table values used in our bSpline interpolation for the x-axis.
		//(we (linear) interpolate between the current and previous tables here)
		for(j=0;j<4;++j)
		{
			tableVals[j] = tables[pos[j]].x * tableInterp;
			tableVals[j] += (prevTables[pos[j]].x * (1.0f-tableInterp));
		}

		//Calculate the value for the x-axis table.
		outVal.x = bSplineInterp(tableVals[0],
								 tableVals[1],
								 tableVals[2],
								 tableVals[3],
								 position-(float)(pos[2]-2));
		outVal.x -= 0.5f;

		//Filter out any dc in the signal.
		dcSpeed.x += (outVal.x - dcPos.x) * 0.000004567f;
		dcPos.x += dcSpeed.x;
		dcSpeed.x *= 0.96f;
		outVal.x -= dcPos.x;

		//The 4 table values used in our bSpline interpolation for the y-axis.
		//(we (linear) interpolate between the current and previous tables here)
		for(j=0;j<4;++j)
		{
			tableVals[j] = tables[pos[j]].y * tableInterp;
			tableVals[j] += (prevTables[pos[j]].y * (1.0f-tableInterp));
		}

		//Calculate the value for the y-axis table.
		outVal.y = bSplineInterp(tableVals[0],
								 tableVals[1],
								 tableVals[2],
								 tableVals[3],
								 position-(float)(pos[2]-2));
		outVal.y -= 0.5f;

		//Filter out any dc in the signal.
		dcSpeed.y += (outVal.y - dcPos.y) * 0.000004567f;
		dcPos.y += dcSpeed.y;
		dcSpeed.y *= 0.96f;
		outVal.y -= dcPos.y;

		//Write the signal to the output.
		tempf = (outVal.x + outVal.y) * fadeOut * fadeIn * 0.5f; //0.5 because there's 2 signals.
		buffer[(i*2)] += tempf * (1.0f-pan);
		buffer[(i*2)+1] += tempf * pan;

		//Update position.
		position += increment;
		position = fmodf(position, (float)TableSize);

		//Update tableInterp.
		if(tableInterp < 1.0f)
			tableInterp += 0.001f;
		else if(tableInterp > 1.0f)
			tableInterp = 1.0f;

		//Update fadeOut.
		if(fadeOut < 1.0f)
		{
			fadeOut -= 0.001f;
			if(fadeOut <= 0.0f)
			{
				active = false;
				break;
			}
		}
		//Update fadeIn.
		if((fadeIn > 0.0f) && (fadeIn < 1.0f))
			fadeIn += 0.001f;
	}
}

//------------------------------------------------------------------------------
void BrushStrokesSynth::setActive(bool val)
{
	int i;

	if(val)
	{
		active = val;
		fadeOut = 1.0f;
		fadeIn = 0.0f;
		for(i=0;i<TableSize;++i)
			tables[i] = 0.0f;
	}
	else if(fadeOut > 0.0f)
		fadeOut -= 0.001f;
}

//------------------------------------------------------------------------------
void BrushStrokesSynth::setFrequency(float val)
{
	frequency = val;
	increment = ((float)TableSize/44100.0f) * frequency;
}

//------------------------------------------------------------------------------
void BrushStrokesSynth::setTables(TwoFloats *val)
{
	int i;
	TwoFloats tempVal;

	tableInterp = 0.0f;

	//Shift tables to prevTables.
	for(i=0;i<(TableSize+4);++i)
		prevTables[i] = tables[i];

	//Copy val to tables.
	for(i=2;i<(TableSize+2);++i)
	{
		tempVal = val[i-2];

		/*This is to scale the values so they lie within 0->1.
		  The 1024 values are arbitrary, used to try and accommodate the
		  fact that the Trails go outside of the window bounds.*/
		tempVal.x = (1024.0f + tempVal.x)/(1280.0f+2048.0f);
		tempVal.y = (1024.0f + tempVal.y)/(720.0f+2048.0f);

		if(tempVal.x < 0.0f)
			fadeOut -= 0.001f;
		else if(tempVal.x > 1.0f)
			fadeOut -= 0.001f;
		if(tempVal.y < 0.0f)
			fadeOut -= 0.001f;
		else if(tempVal.y > 1.0f)
			fadeOut -= 0.001f;

		tables[i] = tempVal;
	}

	/*Fill in the edge cases so we get a smooth transition at the table
	  boundaries with the interpolation. This is not strictly necessary, but
	  it makes things slightly easier.*/
	tables[0] = tables[2 + TableSize-2];
	tables[1] = tables[2 + TableSize-1];
	tables[2 + TableSize] = tables[2];
	tables[2 + TableSize+1] = tables[3];

	if(fadeIn <= 0.0f)
		fadeIn += 0.001f;
}

//------------------------------------------------------------------------------
void BrushStrokesSynth::setPan(float val)
{
	pan = val;
}
