//	SynthSingleton.cpp - Singleton class handling all the synths.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SynthSingleton.h"
#include "SoundHelpers.h"
#include "MiscHelper.h"
#include "GamePlay.h"
#include "Random.h"

using namespace std;

//------------------------------------------------------------------------------
SynthSingleton& SynthSingleton::getInstance()
{
	static SynthSingleton retval;

	return retval;
}

//------------------------------------------------------------------------------
void SynthSingleton::getAudio(float *buffer, int numSamples)
{
	int i;

	if(killVal > 0.0f)
	{
		for(i=0;i<NumBrushStrokes;++i)
		{
			if(brushStrokes[i].getActive())
				brushStrokes[i].getAudio(buffer, numSamples);
		}
		for(i=0;i<NumBells;++i)
		{
			if(bells[i].getActive())
				bells[i].getAudio(buffer, numSamples);
		}
		for(i=0;i<NumWords;++i)
		{
			if(words[i].getActive())
				words[i].getAudio(buffer, numSamples);
		}
		tree.getAudio(buffer, numSamples);

		if(killVal < 1.0f)
		{
			for(i=0;i<numSamples;++i)
			{
				buffer[(i*2)] *= killVal;
				buffer[(i*2)+1] *= killVal;

				killVal -= 0.0001f;
			}
		}
	}

	if(cacophonyOn)
	{
		bool cacophonyFinished = true;

		for(i=0;i<(int)cacophony.size();++i)
		{
			if(cacophonyStart[i] <= 0)
			{
				if(cacophony[i]->getActive())
				{
					cacophony[i]->getAudio(buffer, numSamples);
					cacophonyFinished = false;
				}
			}
			else
			{
				--cacophonyStart[i];
				cacophonyFinished = false;
			}
		}

		if(cacophonyFinished)
		{
			gamePlay->setEnd();
			cacophonyOn = false;
		}

		if(cacophonyFade < 1.0f)
		{
			for(i=0;i<numSamples;++i)
			{
				buffer[(i*2)] *= fakeExp(cacophonyFade);
				buffer[(i*2)+1] *= fakeExp(cacophonyFade);

				cacophonyFade += 0.000005f;
				if(cacophonyFade >= 1.0f)
					break;
			}
		}
	}

	if(activeVoiceover != "")
	{
		if(voiceovers[activeVoiceover]->getActive())
			voiceovers[activeVoiceover]->getAudio(buffer, numSamples);
		else
			activeVoiceover = "";
	}
}

//------------------------------------------------------------------------------
BrushStrokesSynth *SynthSingleton::activateBrushStrokesSynth(int pitchIndex)
{
	int i;
	BrushStrokesSynth *retval = 0;

	for(i=0;i<synthLimit;++i)
	{
		if(!brushStrokes[i].getActive())
		{
			retval = &(brushStrokes[i]);

			retval->setFrequency(SoundHelpers::getFrequencyForPitch(pitchIndex));
			retval->setActive(true);
			break;
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
void SynthSingleton::triggerBell(float frequency, float pan)
{
	int i;

	for(i=0;i<synthLimit;++i)
	{
		if(!bells[i].getActive())
		{
			bells[i].setFrequency(frequency);
			bells[i].setPan(pan);
			bells[i].trigger();
			break;
		}
	}
}

//------------------------------------------------------------------------------
void SynthSingleton::triggerWord(float frequency, float timbre, float pan)
{
	int i;

	for(i=0;i<synthLimit;++i)
	{
		if(!words[i].getActive())
		{
			words[i].setFrequency(frequency, timbre);
			words[i].setPan(pan);
			words[i].trigger();
			break;
		}
	}
}

//------------------------------------------------------------------------------
void SynthSingleton::startTree()
{
	tree.start();
}

//------------------------------------------------------------------------------
void SynthSingleton::stopTree()
{
	tree.stop();
}

//------------------------------------------------------------------------------
void SynthSingleton::killMusic()
{
	killVal -= 0.0001f;
}

//------------------------------------------------------------------------------
void SynthSingleton::triggerVoiceover(const std::string& voiceover)
{
	activeVoiceover = voiceover;
}

//------------------------------------------------------------------------------
void SynthSingleton::cancelVoiceover()
{
	activeVoiceover = "";
}

//------------------------------------------------------------------------------
void SynthSingleton::setGamePlay(GamePlay *gp)
{
	gamePlay = gp;
}

//------------------------------------------------------------------------------
void SynthSingleton::reachedMarker(const std::string& marker)
{
	if(gamePlay)
		gamePlay->markerCallback(marker);
}

//------------------------------------------------------------------------------
void SynthSingleton::startCacophony()
{
	cacophonyOn = true;
}

//------------------------------------------------------------------------------
void SynthSingleton::setSynthLimit(int val)
{

	if((val > -1) && (val < 128))
		synthLimit = val;
}

//------------------------------------------------------------------------------
bool SynthSingleton::run()
{
	int i;
	map<string, StdSharedPtr<Voiceover>::type>::iterator it;

	while(!timeToStop)
	{
		for(it=voiceovers.begin();it!=voiceovers.end();++it)
			it->second->readBufferData();

		for(i=0;i<(int)cacophony.size();++i)
			cacophony[i]->readBufferData();

		//Wait a bit before looping.
		SDL_Delay(62);
	}

	return true;
}

//------------------------------------------------------------------------------
SynthSingleton::SynthSingleton():
gamePlay(0),
activeVoiceover("Intro"),
killVal(1.0f),
cacophonyOn(false),
cacophonyFade(0.0f),
timeToStop(false),
synthLimit(128)
{
	int i;
	Voiceover *v;
	FilePath cacophonyDir = FilePath::getDataFilesDir().getChild("cacophony");

	//Intro voiceover.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("InitialVoiceover.ogg"));
	v->addMarker(857456, "there was a tree");
	v->addMarker(2361937, "she started to sing");
	voiceovers.insert(make_pair("Intro", StdSharedPtr<Voiceover>::type(v)));

	//Outro voiceover.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("FinalVoiceover.ogg"));
	v->addMarker(493024, "in an instant");
	v->addMarker(2132401, "sing for her");
	voiceovers.insert(make_pair("Outro", StdSharedPtr<Voiceover>::type(v)));

	//Letter.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-Letter.ogg"));
	voiceovers.insert(make_pair("Letter", StdSharedPtr<Voiceover>::type(v)));
	//Letter->Trail.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-LetterTrail.ogg"));
	voiceovers.insert(make_pair("LetterTrail", StdSharedPtr<Voiceover>::type(v)));
	//Letter->Trail->Word.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-LetterTrailWord.ogg"));
	voiceovers.insert(make_pair("LetterTrailWord", StdSharedPtr<Voiceover>::type(v)));
	//Letter->Word.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-LetterWord.ogg"));
	voiceovers.insert(make_pair("LetterWord", StdSharedPtr<Voiceover>::type(v)));
	//Letter->Word->Trail.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-LetterWordTrail.ogg"));
	voiceovers.insert(make_pair("LetterWordTrail", StdSharedPtr<Voiceover>::type(v)));

	//Trail.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-Trail.ogg"));
	voiceovers.insert(make_pair("Trail", StdSharedPtr<Voiceover>::type(v)));
	//Trail->Letter.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-TrailLetter.ogg"));
	voiceovers.insert(make_pair("TrailLetter", StdSharedPtr<Voiceover>::type(v)));
	//Trail->Letter->Word.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-TrailLetterWord.ogg"));
	voiceovers.insert(make_pair("TrailLetterWord", StdSharedPtr<Voiceover>::type(v)));
	//Trail->Word.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-TrailWord.ogg"));
	voiceovers.insert(make_pair("TrailWord", StdSharedPtr<Voiceover>::type(v)));
	//Trail->Word->Letter.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-TrailWordLetter.ogg"));
	voiceovers.insert(make_pair("TrailWordLetter", StdSharedPtr<Voiceover>::type(v)));

	//Word.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-Word.ogg"));
	voiceovers.insert(make_pair("Word", StdSharedPtr<Voiceover>::type(v)));
	//Word->Letter.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-WordLetter.ogg"));
	voiceovers.insert(make_pair("WordLetter", StdSharedPtr<Voiceover>::type(v)));
	//Word->Letter->Trail.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-WordLetterTrail.ogg"));
	voiceovers.insert(make_pair("WordLetterTrail", StdSharedPtr<Voiceover>::type(v)));
	//Word->Trail.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-WordTrail.ogg"));
	voiceovers.insert(make_pair("WordTrail", StdSharedPtr<Voiceover>::type(v)));
	//Word->Trail->Letter.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-WordTrailLetter.ogg"));
	voiceovers.insert(make_pair("WordTrailLetter", StdSharedPtr<Voiceover>::type(v)));

	//Waiting voiceover.
	v = new Voiceover(FilePath::getDataFilesDir().getChild("sounds").getChild("Voiceover-Waiting.ogg"));
	voiceovers.insert(make_pair("Waiting", StdSharedPtr<Voiceover>::type(v)));

	///	The various cacophony sounds.
	for(i=0;i<cacophonyDir.getNumChildren();++i)
	{
		v = new Voiceover(cacophonyDir.getChild(i));
		v->setPan(Random::getGlobal().getRandomFloat());
		v->setLevel(0.5f);
		cacophony.push_back(StdSharedPtr<Voiceover>::type(v));

		if(Random::getGlobal().getRandomFloat() > 0.8f)
			cacophonyStart.push_back(0);
		else
			cacophonyStart.push_back((int)(Random::getGlobal().getRandomFloat() * 64.0f));
	}

	startThread(this);
}

//------------------------------------------------------------------------------
SynthSingleton::~SynthSingleton()
{
	timeToStop = true;
	SDL_Delay(300);
}
