//	SynthSingleton.h - Singleton class handling all the synths.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SYNTHSINGLETON_H_
#define SYNTHSINGLETON_H_

#include "BrushStrokesSynth.h"
#include "c0xHeaders.h"
#include "BellSynth.h"
#include "WordSynth.h"
#include "TreeSynth.h"
#include "Voiceover.h"
#include "Thread.h"

#include <map>

class GamePlay;

///	Singleton class handling all the synths.
class SynthSingleton : public Thread
{
  public:
	///	Returns the singleton's sole instance.
	static SynthSingleton& getInstance();

	///	Calculates the audio for all the active synths.
	void getAudio(float *buffer, int numSamples);

	///	Activates and returns a pointer to a BrushStrokesSynth instance.
	/*!
		\param pitchIndex An index to the pitch table we keep in SoundHelpers.
		0->9

		\return All BrushStrokesSynth instances are owned by the SynthSingleton, so
		DO NOT DELETE THIS POINTER.
	 */
	BrushStrokesSynth *activateBrushStrokesSynth(int pitchIndex);

	///	Triggers a bell synth.
	void triggerBell(float frequency, float pan);

	///	Triggers a word synth.
	void triggerWord(float frequency, float timbre, float pan);

	///	Starts the tree singing.
	void startTree();
	///	Stops the tree singing.
	void stopTree();

	///	Kills all the music.
	void killMusic();

	///	Triggers the named Voiceover to play.
	void triggerVoiceover(const std::string& voiceover);
	///	Cancels the currently-active Voiceover.
	void cancelVoiceover();
	///	So we can call GamePlay when a Voiceover reaches a marker.
	void setGamePlay(GamePlay *gp);
	///	Called from Voiceover when it reaches a marker.
	void reachedMarker(const std::string& marker);

	///	Starts the cacophony.
	void startCacophony();

	///	Limits the number of synths we use.
	void setSynthLimit(int val);

	///	Used to stream the audio off the hard disk.
	bool run();
  private:
	///	Constructor.
	SynthSingleton();
	///	Destructor.
	~SynthSingleton();

	///	The number of synths of each type we keep in memory.
	enum
	{
		NumBrushStrokes = 128,
		NumBells = 128,
		NumWords = 128
	};

	///	All our BrushStrokesSynth instances.
	BrushStrokesSynth brushStrokes[NumBrushStrokes];
	///	All our BellSynth instances.
	BellSynth bells[NumBells];
	///	All our WordSynth instances.
	WordSynth words[NumWords];
	///	The TreeSynth.
	TreeSynth tree;
	///	All the various different voiceover files.
	std::map<std::string, StdSharedPtr<Voiceover>::type> voiceovers;
	///	The cacophony at the end.
	std::vector<StdSharedPtr<Voiceover>::type> cacophony;

	///	The currently-active Voiceover.
	std::string activeVoiceover;
	///	So we can call GamePlay when a Voiceover reaches a marker.
	GamePlay *gamePlay;

	///	Used to kill the music.
	float killVal;
	///	Whether or not the cacophony is currently audible.
	bool cacophonyOn;
	///	Used to fade in the cacophony.
	float cacophonyFade;

	///	Used to stagger the start of the various cacophony voices.
	std::vector<int> cacophonyStart;

	///	Thread variable.
	bool timeToStop;

	///	Variable to limit the number of synths we use.
	int synthLimit;
};

#endif
