//	TreeSynth.cpp - A synth representing the Tree's song.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "DebugStatement.h"
#include "TreeSynth.h"
#include "FilePath.h"
#include "Random.h"

#include <vorbis/vorbisfile.h>
#include <cmath>

//------------------------------------------------------------------------------
TreeSynth::TreeSynth():
nextOnset(0),
playing(false),
sineFade(0.0f)
{
	int i;

	for(i=0;i<NumGrains;++i)
		grains[i].active = false;

	sineX0 = 1.0f;
	sineX1 = 0.0f;
	sineCoeff = 2.0f*sinf(3.1415926535897932384626433832795f*58.2705f*2.0f/44100.0f);

	loadSoundFile();
}

//------------------------------------------------------------------------------
TreeSynth::~TreeSynth()
{

}

//------------------------------------------------------------------------------
void TreeSynth::getAudio(float *buffer, int numSamples)
{
	int i, j;
	float amp;
	int index;
	const float loopSizeFloat = (float)loopBufferSize;
	const float sampleTime = 1.0f/loopSizeFloat;

	for(i=0;i<numSamples;++i)
	{
		if(playing)
		{
			//Update nextOnset.
			--nextOnset;
			if(nextOnset <= 0)
			{
				//Activate any new Grains
				for(j=0;j<NumGrains;++j)
				{
					if(!grains[j].active)
					{
						if(Random::getGlobal().getRandomFloat() > 0.5f)
						{
							grains[j].position = 1.0f - sampleTime;
							grains[j].direction = -1.0f;
						}
						else
						{
							grains[j].position = 0.0f;
							grains[j].direction = 1.0f;
						}
						grains[j].pan = Random::getGlobal().getRandomFloat();
						grains[j].active = true;
						break;
					}
				}

				//Update nextOnset.
				nextOnset = (loopBufferSize/8) + (int)(Random::getGlobal().getRandomFloat() * (float)(loopBufferSize/8));
			}
		}

		//Write audio to output buffer.
		for(j=0;j<NumGrains;++j)
		{
			if(grains[j].active)
			{
				//Calculate the amplitude.
				if(grains[j].position < 0.5f)
					amp = grains[j].position * 2.0f;
				else
					amp = (1.0f - grains[j].position) * 2.0f;

				//Calculate the audio.
				index = (int)(grains[j].position * loopSizeFloat);
				buffer[(i*2)] += loopBuffer[index] * amp * (1.0f - grains[j].pan) * (1.0f/(float)(NumGrains-4));
				buffer[(i*2)+1] += loopBuffer[index] * amp * grains[j].pan * (1.0f/(float)(NumGrains-4));

				//Update grain position.
				grains[j].position += sampleTime * grains[j].direction;

				if(grains[j].position <= 0.0f)
					grains[j].active = false;
				else if(grains[j].position >= 1.0f)
					grains[j].active = false;
			}
		}

		//Write sine tone to buffer.
		sineX0 -= (sineCoeff * sineX1);
		sineX1 += (sineCoeff * sineX0);

		buffer[(i*2)] += sineX1 * 0.125f * sineFade;
		buffer[(i*2)+1] += sineX1 * 0.125f * sineFade;

		if(playing && (sineFade < 1.0f))
			sineFade += sampleTime * 2.0f;
		else if(!playing && (sineFade > 0.0f))
			sineFade -= sampleTime * 2.0f;
	}
}

//------------------------------------------------------------------------------
void TreeSynth::start()
{
	playing = true;
}

//------------------------------------------------------------------------------
void TreeSynth::stop()
{
	playing = true;
}

//------------------------------------------------------------------------------
void TreeSynth::loadSoundFile()
{
	int i;
	FILE *file;
	int result;
	float **tempBuf;
	int samplesRead = 0;
	int currentBitstream = 0;
	OggVorbis_File vorbisFile;
	FilePath path = FilePath::getDataFilesDir().getChild("sounds").getChild("TreeSong.ogg");

	file = fopen(path.getPath().c_str(), "rb");

	if(!file)
	{
		DebugStatement("vorbisfile: fopen could not open file: " << path.getPath());
	}
	else
	{
		if(ov_open(file, &vorbisFile, NULL, 0) < 0)
		{
			DebugStatement("vorbisfile: Could not open file: " << path.getPath());
		}
		else
		{
			if(ov_streams(&vorbisFile) > 0)
			{
				DebugStatement("vorbisfile: Loaded file: " << path.getPath());

				//Get the length of the file.
				loopBufferSize = (int)ov_pcm_total(&vorbisFile, 0);

				//Make sure we start from the beginning.
				ov_pcm_seek(&vorbisFile, 0);

				DebugStatement("TreeSynth: loopBufferSize = " << loopBufferSize);

				//Read the file into our buffer.
				while(samplesRead < loopBufferSize)
				{
					result = ov_read_float(&vorbisFile,
										   &tempBuf,
										   loopBufferSize,
										   &currentBitstream);
					if((currentBitstream > 0)||(result <= 0))
					{
						//We've either gone past the end of our bitstream, or there are
						//no other bitstreams in the file, and we've reached the end of
						//the file.
						DebugStatement("TreeSynth: Gone past end of bitstream");
						break;
					}
					else if(result <= loopBufferSize)
					{
						//Copy the audio data to the output buffer.
						for(i=0;i<result;++i)
						{
							if(samplesRead >= loopBufferSize)
								break;
							loopBuffer.push_back(tempBuf[0][i]);
							++samplesRead;
						}
						//DebugStatement("TreeSynth: Copied audio data to loopBuffer: " << samplesRead);
					}
				}
				samplesRead = loopBuffer.size();
				DebugStatement("TreeSynth: Finished while loop.");

				ov_clear(&vorbisFile);
				DebugStatement("TreeSynth: cleared vorbisFile.");
			}
			else
			{
				ov_clear(&vorbisFile);
				DebugStatement("vorbisfile: There are no streams in this file?");
			}
		}
	}
}

