//	Voiceover.cpp - Class representing all the different sections of the voice
//				  over.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Voiceover.h"
#include "DebugStatement.h"
#include "SynthSingleton.h"

using namespace std;

//------------------------------------------------------------------------------
Voiceover::Voiceover(const FilePath& soundFile):
filePosition(0),
active(true),
readBuffer(0),
readPosition(0),
fileReadPosition(0),
pan(0.5f),
level(1.0f)
{
	int i;

	file = fopen(soundFile.getPath().c_str(), "rb");
	if(!file)
	{
		DebugStatement("vorbisfile: fopen could not open file: " << soundFile.getPath());
	}
	else
	{
		if(ov_open(file, &vorbisFile, NULL, 0) < 0)
		{
			DebugStatement("vorbisfile: Could not open file: " << soundFile.getPath());
		}
		else
		{
			if(ov_streams(&vorbisFile) > 0)
			{
				DebugStatement("vorbisfile: Loaded file: " << soundFile.getPath());

				//Fill the buffers.
				for(i=0;i<NumBuffers;++i)
				{
					fillBuffer(i);
					clearedBuffers[i] = false;
				}

				//Get the length of the file.
				length = ov_pcm_total(&vorbisFile, 0);
			}
			else
			{
				ov_clear(&vorbisFile);
				DebugStatement("vorbisfile: There are no streams in this file?");
			}
		}
	}
}

//------------------------------------------------------------------------------
Voiceover::~Voiceover()
{
	if(file)
		ov_clear(&vorbisFile);
}

//------------------------------------------------------------------------------
void Voiceover::getAudio(float *buffer, int numSamples)
{
	int i;
	int tempint;

	if(active)
	{
		//If we've not reached the end of this buffer.
		if((BufferSize-readPosition) >= numSamples)
		{
			for(i=0;i<numSamples;++i)
			{
				buffer[(i*2)] += dataBuffer[readBuffer][readPosition] * (1.0f-pan) * level;
				buffer[(i*2)+1] += dataBuffer[readBuffer][readPosition] * pan * level;

				if(markers.size() > 0)
				{
					if(fileReadPosition >= markers.begin()->first)
					{
						SynthSingleton::getInstance().reachedMarker(markers.begin()->second);
						markers.erase(markers.begin());
					}
				}

				++readPosition;
				++fileReadPosition;
			}
			if(readPosition >= length)
			{
				active = false;
				readPosition = 0;
				clearedBuffers[readBuffer] = true;
				readBuffer = (readBuffer+1)%NumBuffers;
			}
		}
		//If we need to read from the next buffer halfway through this audio block.
		else
		{
			tempint = BufferSize-readPosition;

			//Read the rest of the samples from the current buffer.
			for(i=0;i<tempint;++i)
			{
				buffer[(i*2)] += dataBuffer[readBuffer][readPosition] * (1.0f-pan) * level;
				buffer[(i*2)+1] += dataBuffer[readBuffer][readPosition] * pan * level;

				if(markers.size() > 0)
				{
					if(fileReadPosition >= markers.begin()->first)
					{
						SynthSingleton::getInstance().reachedMarker(markers.begin()->second);
						markers.erase(markers.begin());
					}
				}

				++readPosition;
				++fileReadPosition;
			}

			//Move to the next buffer.
			readPosition = 0;
			clearedBuffers[readBuffer] = true;
			readBuffer = (readBuffer+1)%NumBuffers;
			if(fileReadPosition >= length)
				active = false;

			if(active)
			{
				//Read the remaining samples from the new buffer.
				for(i=tempint;i<numSamples;++i)
				{
					buffer[(i*2)] += dataBuffer[readBuffer][readPosition] * (1.0f-pan) * level;
					buffer[(i*2)+1] += dataBuffer[readBuffer][readPosition] * pan * level;

					if(markers.size() > 0)
					{
						if(fileReadPosition >= markers.begin()->first)
						{
							SynthSingleton::getInstance().reachedMarker(markers.begin()->second);
							markers.erase(markers.begin());
						}
					}

					++readPosition;
					++fileReadPosition;

					if(fileReadPosition >= length)
					{
						active = false;
						break;
					}
				}
			}
		}
		if(fileReadPosition >= length)
			active = false;
	}
}

//------------------------------------------------------------------------------
void Voiceover::setPan(float val)
{
	pan = val;
}

//------------------------------------------------------------------------------
void Voiceover::setLevel(float val)
{
	level = val;
}

//------------------------------------------------------------------------------
void Voiceover::addMarker(int64_t sample, const std::string& name)
{
	markers.insert(make_pair(sample, name));
}

//------------------------------------------------------------------------------
void Voiceover::readBufferData()
{
	int i;

	for(i=0;i<NumBuffers;++i)
	{
		if(clearedBuffers[i])
		{
			fillBuffer(i);
			clearedBuffers[i] = false;
		}
	}
}

//------------------------------------------------------------------------------
void Voiceover::fillBuffer(int buf)
{
	int i;
	int result;
	float **tempBuf;
	int currentBitstream = 0;
	int samplesRead = 0;

	//Make sure we're at the correct point to read from.
	ov_pcm_seek(&vorbisFile, filePosition);

	//Read audio from disk into the current block.
	while(samplesRead < BufferSize)
	{
		result = ov_read_float(&vorbisFile,
							   &tempBuf,
							   (BufferSize-samplesRead),
							   &currentBitstream);
		if((currentBitstream > 0)||(result <= 0))
		{
			//We've either gone past the end of our bitstream, or there are
			//no other bitstreams in the file, and we've reached the end of
			//the file.
			filePosition = 0;
			ov_pcm_seek(&vorbisFile, 0);
		}
		else if(result <= BufferSize)
		{
			//Copy the audio data to the output buffer.
			for(i=0;i<result;++i)
			{
				if(samplesRead >= BufferSize)
					break;
				dataBuffer[buf][samplesRead] = tempBuf[0][i];
				++samplesRead;
				++filePosition;
			}
		}
	}
}
