//	Voiceover.h - Class representing all the different sections of the voice
//				  over.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VOICEOVER_H_
#define VOICEOVER_H_

#include "FilePath.h"

#include <vorbis/vorbisfile.h>
#include <stdint.h>
#include <string>
#include <map>

///	Base class for all the different sections of the voice over.
class Voiceover
{
  public:
	///	Constructor.
	Voiceover(const FilePath& soundFile);
	///	Destructor.
	~Voiceover();

	///	Writes a block of audio.
	void getAudio(float *buffer, int numSamples);
	///	Returns true if the Voiceover is currently active.
	bool getActive() const {return active;};

	///	Sets how the sound should be panned.
	void setPan(float val);
	///	Sets the sound's level.
	void setLevel(float val);

	///	Adds a callback marker to this voiceover.
	void addMarker(int64_t sample, const std::string& name);

	///	Reads data into our buffers (called from a separate thread).
	void readBufferData();
  private:
	///	Helper method to fill a buffer with audio.
	void fillBuffer(int buf);

	///	Some constants.
	enum
	{
		BufferSize = 11025,
		NumBuffers = 3
	};

	///	The sound file to play.
	FILE *file;
	///	The sound file to play.
	OggVorbis_File vorbisFile;
	///	Our read position within the file.
	int64_t filePosition;
	///	The length of the file in samples.
	int64_t length;

	///	If this voiceover is currently active.
	bool active;

	///	Temporary buffers used to stream data from the disk.
	float dataBuffer[NumBuffers][BufferSize];
	///	Which buffer we're currently reading from.
	int readBuffer;
	///	Which buffers need re-loaded.
	bool clearedBuffers[NumBuffers];
	///	Our read position in the active buffer.
	int readPosition;

	///	Our read position within the sound file (as read by getAudio()).
	int64_t fileReadPosition;

	///	How the sound is panned.
	float pan;
	///	The sound's level.
	float level;

	///	Any markers within this sound file.
	std::map<int64_t, std::string> markers;
};

#endif
