//	WordSynth.cpp - An imaginary percussion-ish sound for the Word agents.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "WordSynth.h"
#include "MiscHelper.h"
#include "SoundHelpers.h"
#include "Random.h"

#include <cmath>

//------------------------------------------------------------------------------
WordSynth::WordSynth():
frequency(SoundHelpers::getFrequencyForPitch(0)),
pan(0.5f),
active(false)
{
	float dur = 0.1f; //0.2 second.
	float amp = 1.0f;

	//Randomise dur.
	dur += Random::getGlobal().getRandomFloat() * 0.1f;

	partials[0].duration = dur;
	partials[1].duration = dur * 0.9f;
	partials[2].duration = dur * 0.5f;
	partials[3].duration = dur * 0.25f;
	partials[4].duration = dur * 0.125f;
	partials[5].duration = dur * 0.0625f;
	/*partials[6].duration = dur * 0.25f;
	partials[7].duration = dur * 0.2f;
	partials[8].duration = dur * 0.15f;
	partials[9].duration = dur * 0.1f;
	partials[10].duration = dur * 0.75f;*/

	partials[0].amplitude = amp;
	partials[1].amplitude = amp * 0.67f;
	partials[2].amplitude = amp;
	partials[3].amplitude = amp * 1.8f;
	partials[4].amplitude = amp * 2.67f;
	partials[5].amplitude = amp * 1.67f;
	/*partials[6].amplitude = amp * 1.46f;
	partials[7].amplitude = amp * 1.33f;
	partials[8].amplitude = amp * 1.33f;
	partials[9].amplitude = amp;
	partials[10].amplitude = amp * 1.33f;*/

	//Initialise timbres.
	timbres[0][0] = 1.0f/6.0f;
	timbres[0][1] = 1.0f/2.0f;
	timbres[0][2] = 1.0f;
	timbres[0][3] = 2.89f;
	timbres[0][4] = 4.95f;
	timbres[0][5] = 6.99f;

	timbres[1][0] = 1.0f/6.0f;
	timbres[1][1] = 1.0f/2.0f;
	timbres[1][2] = 1.0f;
	timbres[1][3] = 2.0f;
	timbres[1][4] = 3.01f;
	timbres[1][5] = 4.01f;

	timbres[2][0] = 1.0f/6.0f;
	timbres[2][1] = 1.0f/2.0f;
	timbres[2][2] = 1.0f;
	timbres[2][3] = 3.932f;
	timbres[2][4] = 9.538f;
	timbres[2][5] = 16.688f;

	timbres[3][0] = 1.0f/7.0f;
	timbres[3][1] = 1.0f/2.0f;
	timbres[3][2] = 1.0f;
	timbres[3][3] = 2.77828f;
	timbres[3][4] = 5.18099f;
	timbres[3][5] = 8.16289f;

	timbres[4][0] = 1.0f/6.0f;
	timbres[4][1] = 1.0f/2.0f;
	timbres[4][2] = 1.0f;
	timbres[4][3] = 3.2f;
	timbres[4][4] = 6.23f;
	timbres[4][5] = 6.27f;

	timbres[5][0] = 1.0f/4.0f;
	timbres[5][1] = 1.0f/2.0f;
	timbres[5][2] = 1.0f;
	timbres[5][3] = 1.47f;
	timbres[5][4] = 2.09f;
	timbres[5][5] = 2.56f;

	timbres[6][0] = 1.0f/6.0f;
	timbres[6][1] = 1.0f/2.0f;
	timbres[6][2] = 1.0f;
	timbres[6][3] = 2.572f;
	timbres[6][4] = 4.644f;
	timbres[6][5] = 6.984f;

	timbres[7][0] = 1.0f/6.0f;
	timbres[7][1] = 1.0f/2.0f;
	timbres[7][2] = 1.0f;
	timbres[7][3] = 2.756f;
	timbres[7][4] = 5.423f;
	timbres[7][5] = 8.988f;
}

//------------------------------------------------------------------------------
WordSynth::~WordSynth()
{

}

//------------------------------------------------------------------------------
void WordSynth::getAudio(float *buffer, int numSamples)
{
	int i, j;
	float amp;
	float tempf;
	float midPoint;
	float midPointMult;

	for(i=0;i<numSamples;++i)
	{
		tempf = 0.0f;

		for(j=0;j<NumPartials;++j)
		{
			if(partials[j].ampEnv < partials[j].duration)
			{
				partials[j].x0 -= (partials[j].coeff * partials[j].x1);
				partials[j].x1 += (partials[j].coeff * partials[j].x0);

				midPoint = partials[j].duration * 0.05f;
				midPointMult = (1.0f/(partials[j].duration - midPoint));

				if(partials[j].release)
					amp = fakeLog(1.0f - ((partials[j].ampEnv - midPoint) * (1.0f/(partials[j].duration - midPoint))));
				else 
					amp = fakeLog(partials[j].ampEnv * (1.0f/midPoint));

				tempf += partials[j].x1 * partials[j].amplitude * amp * 0.02f;

				partials[j].ampEnv += (1.0f/44100.0f);
				if(partials[j].ampEnv >= midPoint)
					partials[j].release = true;
			}
		}

		tempf *= amplitude;

		buffer[(i*2)] += tempf * (1.0f-pan);
		buffer[(i*2)+1] += tempf * pan;

		if(partials[0].ampEnv >= partials[0].duration)
			active = false;
	}
}

//------------------------------------------------------------------------------
void WordSynth::trigger()
{
	int i;

	for(i=0;i<NumPartials;++i)
	{
		partials[i].x0 = 1.0f;
		partials[i].x1 = 0.0f;

		partials[i].ampEnv = 0.0f;
		partials[i].release = false;
	}
	amplitude = 0.5f + (0.5f * Random::getGlobal().getRandomFloat());

	active = true;
}

//------------------------------------------------------------------------------
void WordSynth::setFrequency(float val, float timbre)
{
	int i;
	int t = (int)(timbre * ((float)NumTimbres)-0.01f);

	frequency = val * 0.5f;

	frequency += (Random::getGlobal().getRandomFloat() * 4.0f)-2.0f;

	partials[0].frequency = frequency * timbres[t][0];
	partials[1].frequency = frequency * timbres[t][1];
	partials[2].frequency = frequency * timbres[t][2];
	partials[3].frequency = frequency * timbres[t][3];
	partials[4].frequency = frequency * timbres[t][4];
	partials[5].frequency = frequency * timbres[t][5];
	/*partials[6].frequency = frequency * 2.0f;
	partials[7].frequency = frequency * 2.74f;
	partials[8].frequency = frequency * 3.0f;
	partials[9].frequency = frequency * 3.76f;
	partials[10].frequency = frequency * 4.07f;*/

	for(i=0;i<NumPartials;++i)
	{
		partials[i].x0 = 1.0f;
		partials[i].x1 = 0.0f;
		partials[i].coeff = 2.0f*sinf(3.1415926535897932384626433832795f*partials[i].frequency*2.0f/44100.0f);
	}
}

//------------------------------------------------------------------------------
void WordSynth::setPan(float val)
{
	pan = val;
}
